/*!
* @file act_tor.c
*
* @brief TOR interface procedures
*
* @details Actuators procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: act_tor.c 377 2014-04-01#$: FileName
* $Rev:: 377                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-04-01 14:34:00 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_tor.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "dil_tor_out.h"
#include "millipore_types.h"
#include "do_itf.h"
#include "do_def.h"
#include "objects.h"
#include "act_api.h"
#include "act_data.h"
#include "dil_api.h"
#include "dil_iface.h"
#include "canopeniface.h"
#include "sys_softerr.h"

/* Defines -------------------------------------------------------------------*/

/* Declarations --------------------------------------------------------------*/

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/
/*!
 *  @ingroup Actuators_func
 *  @{
 */

/*!
 * @fn ACT_TorSetPower
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @param  actAction  [IN]   ActAction_t  <br>
 *         Action order on actuator
 *
 * @return None
 *
 * @brief  Set power on actuator
 *
 * @details   NA
 *
 */
VOID ACT_TorSetPower(ActType_t actType,ActAction_t actAction)
{
   BOOL isPoweredDoField=FALSE;

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      UINT8 bCmd=0;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
      if(ACT_SWITCH_ON == actAction )
      {
         bCmd= 1u;
         isPoweredDoField=TRUE;
      }
      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_TOR_OUT_SET_SIGNAL ,
                            (DIL_LPVOID)bCmd, sizeof(bCmd), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL UV Control call");
   }
   /* write power status in OD */
   switch(actType)
   {
      case ACT_TYPE_SDS_PUMP_1:
         DO_WRITE(g_DoSdsPump1.IsPowered,isPoweredDoField);
         break;
      case ACT_TYPE_SDS_PUMP_2:
         DO_WRITE(g_DoSdsPump2.IsPowered,isPoweredDoField);
         break;
      case ACT_TYPE_VACUUM_PUMP:
         DO_WRITE(g_DoVaccumPump,isPoweredDoField);
         break;
      case ACT_TYPE_ALARM_REPORT_1:
         DO_WRITE(g_DoAlarmReport.Alarm1,isPoweredDoField);
         break;
      case ACT_TYPE_ALARM_REPORT_2:
         DO_WRITE(g_DoAlarmReport.Alarm2,isPoweredDoField);
         break;
      case ACT_TYPE_TANK_FILLING:
         DO_WRITE(g_DoTankFilling,isPoweredDoField);
         break;
      case ACT_TYPE_UF_TRIGGER:
         DO_WRITE(g_DoUfTrig,isPoweredDoField);
         break;
      case ACT_TYPE_POD_CMD:
         DO_WRITE(g_DoPodCmd,isPoweredDoField);
         break;
      default:
         ASSERT(FALSE,"Wrong actuator type %d",actType);
         break;
   }
   /* Update OD actuator status */
   if(TRUE==isPoweredDoField)
   {
      DO_OR(g_DoActuatorsStatus.ActuatorsStatus , ((UINT16)1<<actType));
   }
   else
   {
      DO_AND(g_DoActuatorsStatus.ActuatorsStatus , (~((UINT16)1<<actType)));
   }
   /* Send PDO */
   canSendPdo(TPDO_ACTUATOR_STATUS);
}


/*!
 * @fn ACT_TorGetState
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return ActState_t
 * @retval
 *
 * @brief  Return actuator state
 *
 * @details   NA
 *
 */
ActState_t ACT_TorGetState(ActType_t actType)
{
   ActState_t actState;
   UINT8 bState=0;
   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_TOR_OUT_GET_STATE ,
                            (DIL_LPVOID)&bState, (UINT32)sizeof(bState), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL TOR get state  call");
   }
   actState=(1u==bState ? ACT_SWITCH_ON : ACT_SWITCH_OFF);
   return actState;
}


/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

