/*!
* @file act_uv.c
*
* @brief UV interface procedures
*
* @details NA
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: act_uv.c 649 2014-11-24 #$: FileName
* $Rev:: 649                     $: Revision of last commit
* $Author:: x157168              $: Author of last commit
* $Date:: 2014-11-24 15:12:56 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_uv.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "dil_uv.h"
#include "millipore_types.h"
#include "do_itf.h"
#include "do_def.h"
#include "objects.h"
#include "act_api.h"
#include "act_data.h"
#include "dil_api.h"
#include "dil_iface.h"
#include "dil_timer.h"
#include "canopeniface.h"
#include "sys_softerr.h"
#include "aaa_api.h"

/*!
 *  @ingroup Actuators_data
 *  @{
 */
/* Defines -------------------------------------------------------------------*/


#define ACT_TYPE_UV_FIRST  (ACT_TYPE_UV_SDS)
#define ACT_TYPE_UV_LAST   (ACT_TYPE_UV_ASM_DUPLEX-ACT_TYPE_UV_SDS+1)


/* Declarations --------------------------------------------------------------*/


/*! Store DO Uv power status location to update */
static BOOL_T * CONST g_ActUvDoVar[]=
{
   &g_DoSdsUv1.IsPowered,
   &g_DoSdsUv2.IsPowered,
   &g_DoAsmUv1.IsPowered,
   &g_DoAsmUv2.IsPowered,
};


/*! Store UV Diag timer handle */
static volatile  DIL_HTIMER_T _ActUvTimerArrayHdl[ACT_TYPE_UV_LAST]={0l,0l,0l,0l};

/*!
 * @}
 */


/* Code ----------------------------------------------------------------------*/
/*!
 *  @ingroup Actuators_func
 *  @{
 */

/*!
 * @fn ACT_UvSetPower
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @param  actAction  [IN]   ActAction_t  <br>
 *         Action order on actuator
 *
 * @return None
 *
 * @brief  Set power on actuator
 *
 * @details   NA
 *
 */
VOID ACT_UvSetPower(ActType_t actType,ActAction_t actAction)
{
   BOOL isPoweredDoField=FALSE;

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      UINT8 bCmd=0;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
      if(ACT_SWITCH_ON == actAction )
      {
         bCmd= 1u;
         isPoweredDoField=TRUE;
      }
      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_UV_CONTROL ,
                            (DIL_LPVOID)bCmd, (UINT32)sizeof(bCmd), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL UV Control call");
   }
   /* write power status in OD */
   DO_WRITE_PTR(g_ActUvDoVar[actType-ACT_TYPE_UV_SDS] , isPoweredDoField);
   /* Update OD actuator status */
   if(TRUE==isPoweredDoField)
   {
      DO_OR(g_DoActuatorsStatus.ActuatorsStatus , ((UINT16)1<<actType));
   }
   else
   {
      DO_AND(g_DoActuatorsStatus.ActuatorsStatus , (~((UINT16)1<<actType)));
   }
   /* Send PDO */
   canSendPdo(TPDO_ACTUATOR_STATUS);
}


/*!
 * @fn ACT_UvGetState
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return ActState_t
 * @retval
 *
 * @brief  Return  actuator state
 *
 * @details   If Actuator 's state differs from the one save , it assert
 *
 */
ActState_t ACT_UvGetState(ActType_t actType)
{
   ActState_t actState;
   UINT8 bState=0;

   if( ACT_TYPE_UV_ASM==actType ||  ACT_TYPE_UV_ASM_DUPLEX==actType)
   {
      /* On V1 process board,  DIL_UV_ASM1, DIL_UV_ASM1 can't be diagnose */
      if(TRUE == *g_ActUvDoVar[actType-ACT_TYPE_UV_SDS])
      {
         actState = ACT_SWITCH_ON;
      }
      else
      {
         actState = ACT_SWITCH_OFF;
      }
   }
   else
   {
      /* If DIL not bypassed, DIL flow rate read */
      if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
      {
         DIL_RC   ret;
         /*retrieve dilHdl */
         DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

         /* call DIL */
         ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_UV_GET_STATE ,
                               (DIL_LPVOID)&bState, (UINT32)sizeof(bState), (DIL_LPUINT32)NULL);
         ASSERT(DIL_GOOD == ret,"Error in DIL UV get state  call");
      }
      actState=(1u==bState ? ACT_SWITCH_ON : ACT_SWITCH_OFF);
   }
   return actState;
}


/*!
 * @return None
 *
 * @brief  Uv diagnostic test
 *
 * @details  Called when UV lamp is switch ON after a delay managed with a dil timer.
 *
 */
VOID ACT_UvDiagnose(VOID *pvData)
{
   BOOL isInDefault=TRUE;
   DIL_HANDLE dilHdl;
   DIL_RC   ret;
   ActType_t actType=(ActType_t)(UINT32)pvData;
   switch(actType)
   {
      case ACT_TYPE_UV_ASM:
      {
         /* retrieve dilHdl */
         dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
         /* call DIL */
         ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_UV_IN_DEFAULT ,
                               (DIL_LPVOID)&isInDefault, (UINT32)sizeof(isInDefault), (DIL_LPUINT32)NULL);
         ASSERT(DIL_GOOD == ret,"Error in DIL UV get default  call");

         if(TRUE==isInDefault)
         {
            /* If default, report */
            /* Update DO actuator diagnotic value to failure */
            g_DoHardwareDiagnostic.ActuatorsDiagResult=((UINT32)1<<actType);
            /* Update UV specific Default information */
            if(FALSE == g_DoAsmUv1.Fault)
            {
               g_DoAsmUv1.Fault=(UINT8)TRUE;
               /* Raise alarm */
               AAA_SetAlarmFromIsr(ALARM_TYPE_ASM_UV1_LAMP);
            }
         }
         else
         {
            /* Update UV specific Default information */
            if(TRUE == g_DoAsmUv1.Fault)
            {
               g_DoAsmUv1.Fault=(UINT8)FALSE;
               /* AAA_ResetAlarm allow reseting pending or active alarm. */
               AAA_ResetAlarmFromIsr(ALARM_TYPE_ASM_UV1_LAMP);
            }
         }
         break;
      }

      case ACT_TYPE_UV_ASM_DUPLEX:
      {
         /* retrieve dilHdl */
         dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
         /* call DIL */
         ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_UV_IN_DEFAULT ,
                               (DIL_LPVOID)&isInDefault, (UINT32)sizeof(isInDefault), (DIL_LPUINT32)NULL);
         ASSERT(DIL_GOOD == ret,"Error in DIL UV get default  call");

         if(TRUE==isInDefault)
         {
            /* If default, report */
            /* Update DO actuator diagnotic value to failure */
            g_DoHardwareDiagnostic.ActuatorsDiagResult=((UINT32)1<<actType);
            /* Update UV specific Default information */
            if(FALSE == g_DoAsmUv2.Fault)
            {
               g_DoAsmUv2.Fault=(UINT8)TRUE;
               /* Raise alarm */
               AAA_SetAlarmFromIsr(ALARM_TYPE_ASM_UV2_LAMP);
            }
         }
         else
         {
            /* Update UV specific Default information */
            if(TRUE == g_DoAsmUv2.Fault)
            {
               g_DoAsmUv2.Fault=(UINT8)FALSE;
               /* AAA_ResetAlarm allow reseting pending or active alarm. */
               AAA_ResetAlarmFromIsr(ALARM_TYPE_ASM_UV2_LAMP);
            }
         }
         break;
      }

      default:
         ERROR("Wrong UV diagnore req (%d)",actType);
         break;
   }

   /* Delete timer. */
   ret = dil_timer_ioctl(  (DIL_UINT32)NULL,
                           IOCTL_CMD_TIMER_DELETE,
                           (DIL_LPVOID)&_ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST],
                           (DIL_UINT32) sizeof(DIL_UINT32),
                           (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL timer deletion");

   _ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST]=0l;
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type.<br>
 *
 * @param dwDelayInSec [IN] UINT8 <br>
 *               Diagnose delay in seconds
 *
 *  @return Diagnose result on actuators list
 *
 *  @brief  Handle diagnose on actuators.
 */
VOID ACT_UvLaunchDelayedDiagnose(ActType_t actType,UINT8 dwDelayInSec)
{
   DIL_RC      ret = DIL_FAILURE;
   DIL_TIMER_T dilTimer;

   ASSERT((ACT_TYPE_UV_ASM==actType ||  ACT_TYPE_UV_ASM_DUPLEX==actType),"UV actuator %d not managed",actType);

   if(0l==_ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST])
   {
      dilTimer.bPeriodic  = 0; // One shot
      dilTimer.htimer     = 0;
      dilTimer.dwDelay    = dwDelayInSec*1000;  // delay in  ms
      dilTimer.pfTimerUserHandler = (timerUserHandler)ACT_UvDiagnose;
      dilTimer.arg= (DIL_LPVOID*) actType;

      /* Create a timer. */
      ret = dil_timer_ioctl(  (DIL_UINT32)NULL,
                              IOCTL_CMD_TIMER_CREATE,
                              (DIL_LPVOID)&dilTimer,
                              (DIL_UINT32) sizeof(DIL_TIMER_T),
                              (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL timer creation");

      if( DIL_GOOD == ret )
      {
         /* Start a timer. */
         ret = dil_timer_ioctl(  (DIL_UINT32)NULL,
                                 IOCTL_CMD_TIMER_START,
                                 (DIL_LPVOID)&dilTimer.htimer,
                                 (DIL_UINT32) sizeof(DIL_UINT32),
                                 (DIL_LPUINT32)NULL);
      }
      ASSERT(DIL_GOOD == ret,"Error in DIL timer start");
      /* Store timer handle */
      _ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST]=dilTimer.htimer;
   }
   else
   {
      /*! It is not sure this shall be consider as an error.
       *  Ie if a second switch on is requested , the second power on will trig a second diagnose request.
       */
      WARNING("A:UV[%d] diag already launch",actType)
   }
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type.<br>
 *
 *  @return None
 *
 *  @brief  Handle diagnose cancelation on UV actuators.
 */
VOID ACT_UvCancelDiagnose(ActType_t actType)
{
   DIL_RC      ret = DIL_FAILURE;

   if(0l != _ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST])
   {
      /* Stop  timer. */
      ret = dil_timer_ioctl(  (DIL_UINT32)NULL,
                              IOCTL_CMD_TIMER_STOP,
                              (DIL_LPVOID)&_ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST],
                              (DIL_UINT32) sizeof(DIL_UINT32),
                              (DIL_LPUINT32)actType);
      ASSERT(DIL_GOOD == ret,"Error in DIL timer stop");
      /* Delete timer. */
      ret = dil_timer_ioctl(  (DIL_UINT32)NULL,
                              IOCTL_CMD_TIMER_DELETE,
                              (DIL_LPVOID)&_ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST],
                              (DIL_UINT32) sizeof(DIL_UINT32),
                              (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL timer deletion");

      _ActUvTimerArrayHdl[actType-ACT_TYPE_UV_FIRST]=0l;
      SYSLOG(INFO,"A:UU Cancel Diag [%d]",actType);
   }
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

