/*!
* @file act_valve.c
*
* @brief Solenoid valve interface procedures
*
* @details NA
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: act_valve.c 377 2014-04-#$: FileName
* $Rev:: 377                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-04-01 14:34:00 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_valve.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "dil_valve.h"
#include "millipore_types.h"
#include "do_itf.h"
#include "do_def.h"
#include "objects.h"
#include "act_api.h"
#include "act_data.h"
#include "dil_api.h"
#include "dil_iface.h"
#include "canopeniface.h"
#include "sys_softerr.h"

/* Defines -------------------------------------------------------------------*/

/* Declarations --------------------------------------------------------------*/

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/
/*!
 *  @ingroup Actuators_func
 *  @{
 */

/*!
 * @fn ACT_ValveSetPower
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @param  actAction  [IN]   ActAction_t  <br>
 *         Action order on actuator
 *
 * @return None
 *
 * @brief  Set power on actuator
 *
 * @details   NA
 *
 */
VOID ACT_ValveSetPower(ActType_t actType,ActAction_t actAction)
{
   BOOL isPoweredDoField=FALSE;

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      UINT8 bCmd=0;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
      if(ACT_SWITCH_ON == actAction )
      {
         bCmd= 1u;
         isPoweredDoField=TRUE;
      }
      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_SOL_VAL_CONTROL ,
                            (DIL_LPVOID)bCmd, (UINT32)sizeof(bCmd), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL VALVE Control call");
   }
   /* write power status in OD */
   switch(actType)
   {
      case ACT_TYPE_EXT_SV_VALVE:
         DO_WRITE(g_DoExternalSolenoidValve,isPoweredDoField);
         break;
      case ACT_TYPE_INLET_SV_VALVE:
         DO_WRITE(g_DoFeedSolenoidValve,isPoweredDoField);
         break;
      case ACT_TYPE_PERMEATE_SV_VALVE_1:
         DO_WRITE(g_DoPermeateSolenoidValve1,isPoweredDoField);
         break;
      case ACT_TYPE_PERMEATE_SV_VALVE_2:
         DO_WRITE(g_DoPermeateSolenoidValve2,isPoweredDoField);
         break;
      case ACT_TYPE_DUMPING_SV_VALVE:
         DO_WRITE(g_DoDumpingSolenoidValve,isPoweredDoField);
         break;
      case ACT_TYPE_RECIRCULATION_SV_VALVE:
         DO_WRITE(g_DoLoopRecirculationValve,isPoweredDoField);
         break;
      default:
         ASSERT(FALSE,"Wrong actuator type %d",actType);
         break;
   }
   /* Update OD actuator status */
   if(TRUE==isPoweredDoField)
   {
      DO_OR(g_DoActuatorsStatus.ActuatorsStatus , ((UINT16)1<<actType));
   }
   else
   {
      DO_AND(g_DoActuatorsStatus.ActuatorsStatus , (~((UINT16)1<<actType)));
   }
   /* Send PDO */
   canSendPdo(TPDO_ACTUATOR_STATUS);
}


/*!
 * @fn ACT_ValveGetState
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return ActState_t
 * @retval
 *
 * @brief  Return  actuator state
 *
 * @details   If Actuator 's state differs from the one save , it assert
 *
 */

ActState_t ACT_ValveGetState(ActType_t actType)
{
   ActState_t actState;
   UINT8 bState=0u;
   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_SOL_VAL_GET_STATE ,
                            (DIL_LPVOID)&bState, (UINT32)sizeof(bState), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL VALVE get state  call");
   }
   actState=(1u==bState ? ACT_SWITCH_ON : ACT_SWITCH_OFF);
   return actState;
}


/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

