/*!
* @file :  app_timer.c
*
* @brief Applicative timer task
*
* @details Trig Timed speaphores to wakeup
*           Measure , Alert and alarm, Proces, Distribution tasks
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: app_timer.c 616 2014-#$:  Filename
* $Rev:: 616                  $: Revision of last commit
* $Author:: x157168           $: Author of last commit
* $Date:: 2014-10-31 15:32:30#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/helper/app_timer.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "millipore_types.h"
#include "os_adaptation.h"
#include "sys_softerr.h"



/*! @ingroup ApplicativeTimer
 *  @{
 */

/* Defintions ----------------------------------------------------------------*/
#define DELAY100MS      ((UINT32) 100)

#define MES_TIM_SEM_IDX ((INT32)0)
#define AAA_TIM_SEM_IDX ((INT32)1)
#define CSM_TIM_SEM_IDX ((INT32)2)
#define ROP_TIM_SEM_IDX ((INT32)3)
#define DIP_TIM_SEM_IDX ((INT32)4)
#define TST_TIM_SEM_IDX ((INT32)5)

/* Declarations --------------------------------------------------------------*/
BOOL g_isAppTimerTaskActive = TRUE;          /*!< TRUE if aplicative timer task
                                                      to activate. */
extern BOOL g_doLaunchApp;
/* Code ----------------------------------------------------------------------*/

/*!
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Applicative Timer task
 *
 *  @details   Timer task run
 *       at differents 100 ms period to trig
 *        -# measure task.
 *        -# alarm task.
 *
 */
ATTRIBUTE_TASK void appTimerTask(void)
{
   eOsRc_t rc;
   static UINT32 _timePeriod = 1u;
   static const PLAINCHAR* _SemaNameArray [7u] =      /*!< Timer name semaphores array */
   {
      "MES_TIM_SEM",
      "AAA_TIM_SEM",
      "CSM_TIM_SEM",
      "ROP_TIM_SEM",
      "DIP_TIM_SEM",
      "TST_TIM_SEM",
      NULL
   };
   static OsHandle_t _hTimerSemaList[ 7u ];           /*!< Timer semaphores handle array */

   rc=OS_SemaLookUpL(_SemaNameArray, _hTimerSemaList );
   ASSERT( OS_GOOD == rc ,"Wrong OS_SemaLookUpL call");

   while (g_isAppTimerTaskActive)
   {
      /*! delay Current Task by 100 ms */
      OS_TaskDelay(DELAY100MS);

      if(TRUE == g_doLaunchApp)
      {

         /*! Trig measure timer */
         OS_SemaSignalH(_hTimerSemaList[MES_TIM_SEM_IDX], OS_REGION_TASK);

         /*! Trig Alert and Alarms timer */
         OS_SemaSignalH(_hTimerSemaList[AAA_TIM_SEM_IDX], OS_REGION_TASK);

         if (0u == (_timePeriod % 5u))
         {
            /*! Every 500 ms Trig Production Process timer */
            OS_SemaSignalH(_hTimerSemaList[ROP_TIM_SEM_IDX], OS_REGION_TASK);

            /*! Every 500 ms Trig Distribution Process timer */
            OS_SemaSignalH(_hTimerSemaList[DIP_TIM_SEM_IDX], OS_REGION_TASK);
         }
         if (0u == (_timePeriod % 20u))
         {
            /* to prevent overload CSM task , timer is reset */
            OS_SemaResetCountH(_hTimerSemaList[CSM_TIM_SEM_IDX]);
            /*! Every 2s ms Trig Consumable timer */
            OS_SemaSignalH(_hTimerSemaList[CSM_TIM_SEM_IDX], OS_REGION_TASK);
         }
         _timePeriod++;
      }
#ifndef NDEBUG
      /* to prevent overload TST task , timer is reset */
      OS_SemaResetCountH(_hTimerSemaList[TST_TIM_SEM_IDX]);
      /*! Trig Test unit timer */
      OS_SemaSignalH(_hTimerSemaList[TST_TIM_SEM_IDX], OS_REGION_TASK);
#endif
   }

   OS_TaskTerminate(OS_SELFTASK);
}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

