/*!
* @file dip_can.c
*
* @brief Distribution Process CanOpen Interface
*
* @details Contain all Distribution stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: dip_can.c 633 2014-11-13#$: Filename
* $Rev:: 633                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-11-13 17:54:08 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_can.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "CanOpenIface.h"
#include "millipore_types.h"
#include "os_adaptation.h"
#include "rtxcapi.h"
#include "dip_api.h"
#include "mes_api.h"
#include "sys_softerr.h"
#include "sys_log.h"
#include "dip_private.h"
#include "do_itf.h"
#include "do_def.h"
#include "act_api.h"

typedef struct DIP_CanLastStatusSend_s
{
   UINT8  ProcessMode;
   UINT8  ProcessState;
   UINT32 SequenceTotalDuration;
   UINT32 SequenceRemainingTime;
} DIP_CanLastStatusSend_t;

/* Declarations --------------------------------------------------------------*/
DIP_ProcessSetpoint_t DistributionProcessSetpoints;
static DIP_CanLastStatusSend_t DistributionLastStatusSend = {(UINT8)255U,(UINT8)0U,(UINT32)0U,(UINT32)0U};
static BOOL isAsmUvLampPowered = FALSE;

static const UCHAR TabDistributionMode[LAST_DIP_MODE][50] = {
   "INITIALIZATION",
   "STANDBY_ALARM",
   "STANDBY_OLD_PACK",
   "STANDBY_NEW_PACK",
   "STANDBY_BYPASS",
   "STANDBY_PACK_OVERRIDE",
   "LISTENING",
   "RUNNING",
   "DUMPING",
   "ALARM_STOP",
   "POLISHER_FLUSH",
   "POLISHER_FLUSH_ALARM_STOP",
   "CAN_NOT_RESTART_POLISHER_FLUSH",
   "RESTART_POLISHER_FLUSH",
   "OPTICAP_FLUSH",
   "OPTICAP_FLUSH_ALARM_STOP",
   "RESTART_OPTICAP_FLUSH",
   "SAN1",
   "SAN1_ALARM_STOP",
   "STARTPACK_TANK_FILLING",
   "STARTPACK_TANK_FILLING_ALARM_STOP",
   "CAN_NOT_RESTART_STARTPACK_TANK_FILLING",
   "RESTART_STARTPACK_TANK_FILLING",
   "TANK_EMPTYING",
   "TANK_EMPTYING_ALARM_STOP",
   "TEST",
   "TEST_ALARM_STOP",
   "TEST_RECIRC",
   "PLEASE WAIT",
   "RESTART_SAN1",
   "CAN_NOT_RESTART_SAN1",
   "SAN2",
   "SAN2_ALARM_STOP",
   "RESTART_SAN2",
   "CAN_NOT_RESTART_SAN2",
   "SAN3",
   "SAN3_ALARM_STOP",
   "RESTART_SAN3",
   "CAN_NOT_RESTART_SAN3",
   "SAN4",
   "SAN4_ALARM_STOP",
   "RESTART_SAN4",
   "CAN_NOT_RESTART_SAN4",
   "SAN STOP",
   "TOC_CLEANING",
   "TOC_CLEANING_ALARM_STOP"
};


static const UCHAR TabDistributionState[LAST_DIP_STATE][35] = {
   "WAIT",
   "RECIRCULATION",
   "RECIRCULATION_MEASURE",
   "DISTRIBUTION_MEASURE",
   "DEPRESSURIZATION",
   "DUMP",
   "START_KIT_DUMP",
   "START_KIT_FILL_TANK",
   "WAIT TEST",
   "ENTERING_SEQUENCE"};

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/

/*!
 *  @brief  Record Diagnostic Measure
 *
 *  @details  NA
 *
 */
VOID dip_recordDiagnosticMeasure (VOID)
{
    DO_WRITE(g_DoAlarmDiagnosticReport.DistributionPressure, (REAL32)(MES_GetMeasure(MES_DISTRIBUTION_PRESSURE)));
}

VOID dip_readCanOpenSetpoints (VOID)
{
   DO_READ(DistributionProcessSetpoints.StartDistributionPressure, g_DoDistributionSetpoints.StartDistributionPressure);
   DO_READ(DistributionProcessSetpoints.StopDistributionPressure, g_DoDistributionSetpoints.StopDistributionPressure);
   DO_READ(DistributionProcessSetpoints.AutoRecirculationDuration, g_DoDistributionSetpoints.AutoRecirculationDuration);
   DO_READ(DistributionProcessSetpoints.PostDistributionRecirculationDuration, g_DoDistributionSetpoints.PostDistributionRecirculationDuration);
   DO_READ(DistributionProcessSetpoints.AutoDumpOptionActivation, g_DoDistributionSetpoints.AutoDumpOptionActivation);
   DO_READ(DistributionProcessSetpoints.TocMeasureRequested, g_DoDistributionSetpoints.TOCMeasureRequested);
}

VOID dip_readFirstStartFlag (VOID)
{
   DO_READ(DistributionProcessParameters.pProcessRecord->isFirstStartUp, g_DoDistributionSetpoints.IsTheFirstStartUpDist);
   SYS_RefreshBackupRecord(SYS_REC_DIST);
}

VOID dip_informCanOpenTocMeasureStarted (VOID)
{
   DO_WRITE(g_DoDistributionSetpoints.TOCMeasureRequested, FALSE);
   DistributionProcessSetpoints.TocMeasureRequested = FALSE;
}

VOID DIP_CanUpdateProcessStatus (DIP_ProcessParam_t * p_ProcessParam)
{
   DipMode_e l_ProcessMode = p_ProcessParam->ProcessDescription.ProcessMode;
   BOOL isAsmUvLampMustBePowered;
   
   if ((DIP_SEQUENCE_STANDBY == p_ProcessParam->ProcessDescription.ProcessSequence) &&
       (DIP_STATE_DEPRESSURIZATION == p_ProcessParam->ProcessDescription.ProcessState))
   {
      l_ProcessMode = DIP_MODE_PLEASE_WAIT; // Only for the MMI
   }
      
   if(DistributionLastStatusSend.SequenceTotalDuration != p_ProcessParam->ProcessDescription.SequenceTotalDuration)
   {
      DistributionLastStatusSend.SequenceTotalDuration = (UINT32)(p_ProcessParam->ProcessDescription.SequenceTotalDuration);
      DO_WRITE(g_DoDistributionControl.SequenceTotalDuration, DistributionLastStatusSend.SequenceTotalDuration);
   }
      
   if(DistributionLastStatusSend.SequenceRemainingTime != p_ProcessParam->ProcessDescription.SequenceRemainingTime)
   {
      DistributionLastStatusSend.SequenceRemainingTime = (UINT32)(p_ProcessParam->ProcessDescription.SequenceRemainingTime);
      DO_WRITE(g_DoDistributionControl.SequenceRemainingTime, DistributionLastStatusSend.SequenceRemainingTime);
   }

   if((DistributionLastStatusSend.ProcessMode != l_ProcessMode)   ||
      (DistributionLastStatusSend.ProcessState != p_ProcessParam->ProcessDescription.ProcessState))
   {
      DistributionLastStatusSend.ProcessMode = (UINT8)(l_ProcessMode);
      DistributionLastStatusSend.ProcessState = (UINT8)(p_ProcessParam->ProcessDescription.ProcessState);
      DO_WRITE(g_DoDistributionControl.Mode, DistributionLastStatusSend.ProcessMode);
      DO_WRITE(g_DoDistributionControl.ActuatorsState, DistributionLastStatusSend.ProcessState);
      
      /*! Trig PDO send */
      canSendPdo(TPDO_DIST_PROCESS_STATUS);
      SYSLOG(TEST,"Distribution Process: Mode:%s, State:%s, Time:%d, Total:%d, clock:%d",
             TabDistributionMode[DistributionLastStatusSend.ProcessMode],
             TabDistributionState[DistributionLastStatusSend.ProcessState],
             DistributionLastStatusSend.SequenceRemainingTime,
             DistributionLastStatusSend.SequenceTotalDuration,
             TMC_StartTimeCounter());
   }
   
   /* Update ASM UV Lamp Status */
   if ((DIP_MODE_INITIALIZATION        == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN1                  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN1_ALARM_STOP       == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_RESTART_SAN1          == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_CAN_NOT_RESTART_SAN1  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN2                  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN2_ALARM_STOP       == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_RESTART_SAN2          == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_CAN_NOT_RESTART_SAN2  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN3                  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN3_ALARM_STOP       == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_RESTART_SAN3          == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_CAN_NOT_RESTART_SAN3  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN4                  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN4_ALARM_STOP       == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_RESTART_SAN4          == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_CAN_NOT_RESTART_SAN4  == p_ProcessParam->ProcessDescription.ProcessMode) ||
       (DIP_MODE_SAN_STOP              == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      /* If the process is initializing or in Sanitization, the ASM UVL Lamp shall be OFF */
      if (TRUE == isAsmUvLampPowered)
      {
         ACT_SetPower(ACT_TYPE_UV_ASM, ACT_SWITCH_OFF);
         isAsmUvLampPowered = FALSE;
      }
   }
   else
   {
      /* In other modes, the ASM UV Lamp is controlled by the CanOpen Object set by MMI */
      DO_READ(isAsmUvLampMustBePowered, g_DoAsmUv1.IsPowered);
      if (TRUE == isAsmUvLampMustBePowered)
      {
         if (FALSE == isAsmUvLampPowered)
         {
            ACT_SetPower(ACT_TYPE_UV_ASM, ACT_SWITCH_ON);
            isAsmUvLampPowered = TRUE;
         }
      }
      else
      {
         if (TRUE == isAsmUvLampPowered)
         {
            ACT_SetPower(ACT_TYPE_UV_ASM, ACT_SWITCH_OFF);
            isAsmUvLampPowered = FALSE;
         }
      }
   }
}


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

