/*!
* @file dip_state_machine.c
*
* @brief Distribution Task
*
* @details Contain the state machine of the DIstribution Process
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: dip_state_machine.c 683 #$: Filename
* $Rev:: 683                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-12-22 11:36:49 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_state_machine.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "dip_api.h"
#include "dip_rec.h"
#include "dip_private.h"
#include "sys_softerr.h"
#include "dip_data.h"
#include "mes_api.h"
#include "aaa_api.h"
#include "csm_api.h"
#include "sys_log.h"
#include "dip_api.h"
#include "system_parameters.h"
#include "do_itf.h"
#include "do_def.h"

/* Declarations --------------------------------------------------------------*/
static DipSequence_e dip_getAssociatedSequence (DipMode_e DipMode);
static VOID dip_manageOrder(DIP_ProcessParam_t * p_ProcessParam, DipOrder_e NewOrder);
static VOID dip_manageAlarmStop(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_managePolisherStatus(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageStartPackTankTransition(DIP_ProcessParam_t * p_ProcessParam);

DIP_ProcessParam_t DistributionProcessParameters;

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the state machine
 *
 *  @details
 *
 */
VOID dip_distributionStateMachine(DipOrder_e Order)
{
   BOOL ReEnterMode=TRUE;

   if (DIP_LAST_ORDER != Order)
   {
      dip_manageOrder(&DistributionProcessParameters, Order);
   }

   while (TRUE == ReEnterMode)
   {
      DistributionProcessParameters.ProcessDescription.isEnteringMode = FALSE;
      dip_manageAlarmStop(&DistributionProcessParameters);
      dip_managePolisherStatus(&DistributionProcessParameters);
      dip_manageSequence(&DistributionProcessParameters);
      dip_manageStartPackTankTransition(&DistributionProcessParameters);
      ReEnterMode = DistributionProcessParameters.ProcessDescription.isEnteringMode;
   }
   dip_manageTransition(&(DistributionProcessParameters.TransitionDescription));
   DIP_CanUpdateProcessStatus(&DistributionProcessParameters);
}

/*!
 *  @param DipMode [IN] DipMode_e <br>
 *                    Process Mode
 *
 *  @return DipSequence_e : The associated sequence
 *
 *  @brief  Get the associated sequence of a process mode
 *
 *  @details   Allow the process to know which sequence to apply in a certain mode
 *
 */
static DipSequence_e dip_getAssociatedSequence (DipMode_e DipMode)
{
   DipSequence_e AssociatedSequence=DIP_SEQUENCE_STANDBY;
   switch(DipMode)
   {
      case DIP_MODE_INITIALIZATION:                         {AssociatedSequence=DIP_SEQUENCE_INITIALIZATION; break;}
      case DIP_MODE_STANDBY_ALARM:                          {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_STANDBY_OLD_PACK:                       {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_STANDBY_NEW_PACK:                       {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_STANDBY_BYPASS:                         {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_STANDBY_PACK_OVERRIDE:                  {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_LISTENING:                              {AssociatedSequence=DIP_SEQUENCE_LISTENING; break;}
      case DIP_MODE_RUNNING:                                {AssociatedSequence=DIP_SEQUENCE_RUNNING; break;}
      case DIP_MODE_DUMPING:                                {AssociatedSequence=DIP_SEQUENCE_DUMPING; break;}
      case DIP_MODE_ALARM_STOP:                             {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_POLISHER_FLUSH:                         {AssociatedSequence=DIP_SEQUENCE_POLISHER_FLUSH; break;}
      case DIP_MODE_POLISHER_FLUSH_ALARM_STOP:              {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_CAN_NOT_RESTART_POLISHER_FLUSH:         {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_RESTART_POLISHER_FLUSH:                 {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_OPTICAP_FLUSH:                          {AssociatedSequence=DIP_SEQUENCE_OPTICAP_FLUSH; break;}
      case DIP_MODE_OPTICAP_FLUSH_ALARM_STOP:               {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_RESTART_OPTICAP_FLUSH:                  {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_SAN1:                                   {AssociatedSequence=DIP_SEQUENCE_SAN1; break;}
      case DIP_MODE_SAN1_ALARM_STOP:                        {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_STARTPACK_TANK_FILLING:                 {AssociatedSequence=DIP_SEQUENCE_STARTPACK_TANK_FILLING; break;}
      case DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP:      {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING: {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_RESTART_STARTPACK_TANK_FILLING:         {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_TANK_EMPTYING:                          {AssociatedSequence=DIP_SEQUENCE_TANK_EMPTYING; break;}
      case DIP_MODE_TANK_EMPTYING_ALARM_STOP:               {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_TEST:                                   {AssociatedSequence=DIP_SEQUENCE_TEST; break;}
      case DIP_MODE_TEST_ALARM_STOP:                        {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_TEST_RECIRC:                            {AssociatedSequence=DIP_SEQUENCE_TEST_RECIRC; break;}
      case DIP_MODE_RESTART_SAN1:                           {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_CAN_NOT_RESTART_SAN1:                   {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_SAN2:                                   {AssociatedSequence=DIP_SEQUENCE_SAN2; break;}
      case DIP_MODE_SAN2_ALARM_STOP:                        {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_RESTART_SAN2:                           {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_CAN_NOT_RESTART_SAN2:                   {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_SAN3:                                   {AssociatedSequence=DIP_SEQUENCE_SAN3; break;}
      case DIP_MODE_SAN3_ALARM_STOP:                        {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_RESTART_SAN3:                           {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_CAN_NOT_RESTART_SAN3:                   {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_SAN4:                                   {AssociatedSequence=DIP_SEQUENCE_SAN4; break;}
      case DIP_MODE_SAN4_ALARM_STOP:                        {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      case DIP_MODE_RESTART_SAN4:                           {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_CAN_NOT_RESTART_SAN4:                   {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_SAN_STOP:                               {AssociatedSequence=DIP_SEQUENCE_STANDBY; break;}
      case DIP_MODE_TOC_CLEANING:                           {AssociatedSequence=DIP_SEQUENCE_TOC_CLEANING; break;}
      case DIP_MODE_TOC_CLEANING_ALARM_STOP:                {AssociatedSequence=DIP_SEQUENCE_ALARM_STOP; break;}
      default:
      {
         ERROR("Dip mode error %d",DipMode);
         break;
      }
   }

   return(AssociatedSequence);
}


/*!
 *  @param DipMode [IN] DipMode_e <br>
 *                    Process Mode
 *
 *  @param pDipModeNominal [OUT] DipMode_e * <br>
 *                    Process Mode
 *
 *  @param pDipModeAlarmStop [OUT] DipMode_e * <br>
 *                    Process Mode
 *
 *  @return BOOL : If the requested mode have associated modes (because some modes do not have associated modes)
 *
 *  @brief  Get the associated mode (Nominal, AlarmStop) to a certain mode
 *
 *  @details   Allow the process to know which modes are associated (Nominal, AlarmStop)
 *
 */
BOOL dip_getAssociatedMode (DipMode_e DipMode,
                            DipMode_e * pDipModeNominal,
                            DipMode_e * pDipModeAlarmStop)
{
   BOOL isModeFoundReturn = TRUE;

   * pDipModeNominal = LAST_DIP_MODE;
   * pDipModeAlarmStop = LAST_DIP_MODE;

   switch(DipMode)
   {
      case DIP_MODE_LISTENING:
      case DIP_MODE_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_LISTENING;
         * pDipModeAlarmStop = DIP_MODE_ALARM_STOP;
         break;
      }

      case DIP_MODE_RUNNING:
      {
         * pDipModeNominal = DIP_MODE_RUNNING;
         * pDipModeAlarmStop = DIP_MODE_ALARM_STOP;
         break;
      }

      case DIP_MODE_DUMPING:
      {
         * pDipModeNominal = DIP_MODE_DUMPING;
         * pDipModeAlarmStop = DIP_MODE_ALARM_STOP;
         break;
      }

      case DIP_MODE_POLISHER_FLUSH:
      case DIP_MODE_POLISHER_FLUSH_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_POLISHER_FLUSH;
         * pDipModeAlarmStop = DIP_MODE_POLISHER_FLUSH_ALARM_STOP;
         break;
      }

      case DIP_MODE_OPTICAP_FLUSH:
      case DIP_MODE_OPTICAP_FLUSH_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_OPTICAP_FLUSH;
         * pDipModeAlarmStop = DIP_MODE_OPTICAP_FLUSH_ALARM_STOP;
         break;
      }

      case DIP_MODE_SAN1:
      case DIP_MODE_SAN1_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_SAN1;
         * pDipModeAlarmStop = DIP_MODE_SAN1_ALARM_STOP;
         break;
      }

      case DIP_MODE_STARTPACK_TANK_FILLING:
      case DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_STARTPACK_TANK_FILLING;
         * pDipModeAlarmStop = DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP;
         break;
      }

      case DIP_MODE_TANK_EMPTYING:
      case DIP_MODE_TANK_EMPTYING_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_TANK_EMPTYING;
         * pDipModeAlarmStop = DIP_MODE_TANK_EMPTYING_ALARM_STOP;
         break;
      }

      case DIP_MODE_TEST:
      case DIP_MODE_TEST_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_TEST;
         * pDipModeAlarmStop = DIP_MODE_TEST_ALARM_STOP;
         break;
      }

      case DIP_MODE_TEST_RECIRC:
      {
         * pDipModeNominal = DIP_MODE_TEST_RECIRC;
         * pDipModeAlarmStop = DIP_MODE_TEST_ALARM_STOP;
         break;
      }

      case DIP_MODE_SAN2:
      case DIP_MODE_SAN2_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_SAN2;
         * pDipModeAlarmStop = DIP_MODE_SAN2_ALARM_STOP;
         break;
      }

      case DIP_MODE_SAN3:
      case DIP_MODE_SAN3_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_SAN3;
         * pDipModeAlarmStop = DIP_MODE_SAN3_ALARM_STOP;
         break;
      }

      case DIP_MODE_SAN4:
      case DIP_MODE_SAN4_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_SAN4;
         * pDipModeAlarmStop = DIP_MODE_SAN4_ALARM_STOP;
         break;
      }

      case DIP_MODE_TOC_CLEANING:
      case DIP_MODE_TOC_CLEANING_ALARM_STOP:
      {
         * pDipModeNominal = DIP_MODE_TOC_CLEANING;
         * pDipModeAlarmStop = DIP_MODE_TOC_CLEANING_ALARM_STOP;
         break;
      }

      default:
      {
         isModeFoundReturn = FALSE;
         break;
      }
   }
   return (isModeFoundReturn);
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewMode [OUT] DipMode_e <br>
 *                    New Process Mode
 *
 *  @param isRecorded [OUT] BOOL <br>
 *                    Is the New Process Mode shall be recorded
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Set a new mode
 *
 *  @details: If the new mode is different from the old one, the parameters are reset
 *            and if the associated sequence is different, the process re-enter a new sequence
 *
 */
VOID dip_setNewMode(DIP_ProcessParam_t * p_ProcessParam, DipMode_e NewMode, BOOL isRecorded)
{
   if(NewMode != p_ProcessParam->ProcessDescription.ProcessMode)
   {
      DipSequence_e AssociatedSequence = dip_getAssociatedSequence(NewMode);
      p_ProcessParam->ProcessDescription.ProcessMode = NewMode;
      p_ProcessParam->ProcessDescription.isEnteringMode = TRUE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      if (AssociatedSequence != p_ProcessParam->ProcessDescription.ProcessSequence)
      {
         p_ProcessParam->ProcessDescription.ProcessSequence = AssociatedSequence;
         p_ProcessParam->ProcessDescription.isEnteringSequence = TRUE;
      }
      if (TRUE == isRecorded)
      {
         p_ProcessParam->pProcessRecord->InterruptedProcessMode = NewMode;
         SYS_RefreshBackupRecord(SYS_REC_DIST);
      }
   }
}

/*!
 *  @param DipState [IN] DipState_e <br>
 *                    Process State
 *
 *  @return DipTransition_e : The associated transition
 *
 *  @brief  Get the associated transition to switch to a process state
 *
 *  @details   Allow the process to know which transition to apply to a certain state
 *
 */
DipTransition_e dip_getAssociatedTransition (DipState_e DipState)
{
   DipTransition_e AssociatedTransition = DIP_TRANSITION_TO_WAIT;
   switch(DipState)
   {
      case DIP_STATE_WAIT:                   {AssociatedTransition=DIP_TRANSITION_TO_WAIT;             break;}
      case DIP_STATE_RECIRCULATION:          {AssociatedTransition=DIP_TRANSITION_TO_SAN_RECIRCULATION;break;}
      case DIP_STATE_RECIRCULATION_MEASURE:  {AssociatedTransition=DIP_TRANSITION_TO_RECIRCULATION;    break;}
      case DIP_STATE_DISTRIBUTION_MEASURE:   {AssociatedTransition=DIP_TRANSITION_TO_RECIRCULATION;    break;}
      case DIP_STATE_DEPRESSURIZATION:       {AssociatedTransition=DIP_TRANSITION_TO_DEPRESSURIZATION; break;}
      case DIP_STATE_DUMP:                   {AssociatedTransition=DIP_TRANSITION_TO_DUMP;             break;}
      case DIP_STATE_START_KIT_DUMP:         {AssociatedTransition=DIP_TRANSITION_TO_START_KIT_DUMP;   break;}
      case DIP_STATE_START_KIT_FILL_TANK:    {AssociatedTransition=DIP_TRANSITION_TO_START_KIT_FILL_TANK; break;}
      case DIP_STATE_WAIT_TEST:              {AssociatedTransition=DIP_TRANSITION_TO_WAIT_TEST;        break;}
      default:
      {
         AssociatedTransition=DIP_TRANSITION_TO_WAIT;
         ERROR("Wrong DipState %d",DipState);
         break;
      }
   }
   return(AssociatedTransition);
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewOrder [OUT] DipOrder_e <br>
 *                    Order
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage a order send to the process
 *
 *  @details: The order depends on the process mode
 *
 */
static VOID dip_manageOrder(DIP_ProcessParam_t * p_ProcessParam, DipOrder_e NewOrder)
{
   switch(NewOrder)
   {
      case DIP_ORDER_GOTO_STANDBY:
      {
         if(DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            /* Special case because we need te depressurized after a startpack Tank Filling */
            dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
            dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else if((DIP_MODE_LISTENING == p_ProcessParam->ProcessDescription.ProcessMode)        ||
                 (DIP_MODE_RUNNING == p_ProcessParam->ProcessDescription.ProcessMode)          ||
                 (DIP_MODE_DUMPING == p_ProcessParam->ProcessDescription.ProcessMode)          ||
                 (DIP_MODE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)       ||
                 (DIP_MODE_TANK_EMPTYING == p_ProcessParam->ProcessDescription.ProcessMode)    ||
                 (DIP_MODE_TANK_EMPTYING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else if((DIP_MODE_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_POLISHER_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_POLISHER_FLUSH, TRUE);
         }
         else if((DIP_MODE_OPTICAP_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_OPTICAP_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_OPTICAP_FLUSH, TRUE);
         }
         else if((DIP_MODE_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_STARTPACK_TANK_FILLING, TRUE);
         }
         else if((DIP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_SAN1_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN1, TRUE);
         }
         else if((DIP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_SAN2_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN2, TRUE);
         }
         else if((DIP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_SAN3_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN3, TRUE);
         }
         else if((DIP_MODE_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_SAN4_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN4, TRUE);
         }
         else if((DIP_MODE_TOC_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_TOC_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else if((DIP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (DIP_MODE_TEST_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (DIP_MODE_TEST_RECIRC == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if ((p_ProcessParam->pProcessRecord->InterruptedProcessMode == DIP_MODE_RESTART_STARTPACK_TANK_FILLING) ||
                (p_ProcessParam->pProcessRecord->InterruptedProcessMode == DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING))
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING, TRUE);
            }
            else
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
            }
         }
         else
         {
            /* Do Nothing */
         }
         break;
      }

      case DIP_ORDER_GOTO_READY:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
            }
         }
         break;
      }

      case DIP_ORDER_START_POLISHER_FLUSH:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_STANDBY_NEW_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (DIP_MODE_RESTART_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_POLISHER_FLUSH, TRUE);
               CSM_WriteInstallDate(CSM_POLISHER,FALSE);
               // Disable the check new pack for a short delay
               p_ProcessParam->ProcessDescription.isCheckNewPackDisable = TRUE;
            }
         }
         break;
      }

      case DIP_ORDER_START_OPTICAP_FLUSH:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_RESTART_OPTICAP_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_OPTICAP_FLUSH, TRUE);
            }
         }
         break;
      }

      case DIP_ORDER_START_SAN:
      {
         if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
         {
            if((DIP_MODE_STANDBY_BYPASS == p_ProcessParam->ProcessDescription.ProcessMode) ||
               (DIP_MODE_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode))
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN1, TRUE);
            }
            else if(DIP_MODE_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN2, TRUE);
            }
            else if(DIP_MODE_RESTART_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN3, TRUE);
            }
            else if(DIP_MODE_RESTART_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN4, TRUE);
            }
            else if(DIP_MODE_SAN_STOP == p_ProcessParam->ProcessDescription.ProcessMode)
            {
               dip_numberOfSanCycle = 0;
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN4, TRUE);
            }
         }
         break;
      }

      case DIP_ORDER_START_PACK_TANK_FILLING:
      {
         if(DIP_MODE_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_STARTPACK_TANK_FILLING, TRUE);
               // Disable the check new pack for a short delay
               p_ProcessParam->ProcessDescription.isCheckNewPackDisable = TRUE;
            }
         }
         break;
      }

      case DIP_ORDER_START_TANK_EMPTYING:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)       ||
            (DIP_MODE_STANDBY_BYPASS== p_ProcessParam->ProcessDescription.ProcessMode)          ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TANK_EMPTYING, TRUE);
            }
         }
         break;
      }

      case DIP_ORDER_GOTO_TEST_MODE:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)       ||
            (DIP_MODE_STANDBY_BYPASS== p_ProcessParam->ProcessDescription.ProcessMode)          ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode)  ||
            (DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING== p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
            }
         }
         break;
      }

      case DIP_ORDER_START_RECIRC_TEST:
      {
         if(DIP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TEST_RECIRC, FALSE);
            }
         }
         break;
      }
      case DIP_ORDER_STOP_TEST:
      {
         if(DIP_MODE_TEST_RECIRC == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
            }
         }
         break;
      }
      case DIP_ORDER_SKIP_SEQUENCE:
      {
         if((DIP_MODE_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_OPTICAP_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_TOC_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
         }
         else if((DIP_MODE_LISTENING == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (DIP_MODE_RUNNING == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            DistributionSkipSequence = TRUE;
         }
         else if(DIP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_SAN2, TRUE);
         }
         else if(DIP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_SAN3, TRUE);
         }
         else if(DIP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_SAN4, TRUE);
         }
         break;
      }
      case DIP_ORDER_EXIT_SAN:
      {
         if((DIP_MODE_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_SAN4_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            dip_numberOfSanCycle = 0;
            dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_BYPASS, TRUE);
         }
         break;
      }

      case DIP_ORDER_START_TOC_CLEANING:
      {
         if((DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TOC_CLEANING, TRUE);
            }
         }
         break;
      }

      default:
      {
         ASSERT(FALSE, "Unkown Distribution process command: %d", NewOrder);
         break;
      }
   }
}

/*!
 *  @param includeTankEmpty [IN] BOOL <br>
 *                    is the TAnk Empty included in the check
 *
 *  @param includePackOut [IN] BOOL <br>
 *                    is the Pack Out alarm stop included in the check
 *  @return BOOL
 *  @retval TRUE if an alarm stop is present
 *  @retval FALSE if no alarm stop is present
 *
 *  @brief  Check if there is an alarm stop present
 *
 *  @details: The check can include Tank Empty alarm or not and
 *            The check can include Polisher Pack Out alarm or not
 *
 */
BOOL dip_isAlarmStopPresent(BOOL includePackOut, BOOL includeTankEmpty)
{
   BOOL isAlarmStopPresent = FALSE;

   isAlarmStopPresent =((AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_DISTRIB_PRESSURE_SENSOR_ERROR))||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_DISTRIB_PRESSURE_MAX))         ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))            ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_SDS_LEAK_DETECTOR))        ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_DISCONNECTED))       ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_POLISHER_INCORRECT))      ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_POLISHER_INVALID)));

   if (TRUE == includeTankEmpty)
   {
      isAlarmStopPresent = (isAlarmStopPresent                                            ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_EMPTY)));
   }

   if (TRUE == includePackOut)
   {
      isAlarmStopPresent = (isAlarmStopPresent                                      ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_POLISHER_OUT)));
   }

   return (isAlarmStopPresent);
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Check if there is an alarm stop or a pause to switch the process mode
 *
 *  @details: The order depends on the process mode
 *
 */
static VOID dip_manageAlarmStop(DIP_ProcessParam_t * p_ProcessParam)
{
   BOOL isAlarmPresent = FALSE;
   BOOL isAlarmOtherThanTankEmptyPresent = FALSE;
   DipMode_e DipModeNominal;
   DipMode_e DipModeAlarmStop;
   BOOL isModeFound = dip_getAssociatedMode (p_ProcessParam->ProcessDescription.ProcessMode, &DipModeNominal, &DipModeAlarmStop);

   isAlarmPresent = dip_isAlarmStopPresent(TRUE, TRUE);

   if (TRUE == isModeFound)
   {
      if(TRUE == isAlarmPresent)
      {
         if((DIP_MODE_TANK_EMPTYING == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (DIP_MODE_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            /* For DIP_MODE_TANK_EMPTYING, DIP_MODE_STARTPACK_TANK_FILLING and in SANITIZATION, "Tank Empty"  is not considered as an alarm stop */
            isAlarmOtherThanTankEmptyPresent = dip_isAlarmStopPresent(TRUE, FALSE);

            if(TRUE == isAlarmOtherThanTankEmptyPresent)
            {
               // There is an alarm stop (excepted tank empty) so process shall switches to the alarm mode
               dip_recordDiagnosticMeasure();
               dip_setNewMode(p_ProcessParam, DipModeAlarmStop, FALSE);
            }
         }
         else if(p_ProcessParam->ProcessDescription.ProcessMode == DipModeNominal)
         {
            // There is an alarm stop so process shall switches the alarm mode
            dip_recordDiagnosticMeasure();
            if (DIP_MODE_TEST_ALARM_STOP != DipModeAlarmStop)
            {
               dip_setNewMode(p_ProcessParam, DipModeAlarmStop, TRUE);
            }
            else
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_TEST_ALARM_STOP, FALSE);
            }
         }
         else
         {
            /* Do Nothing */
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the tank level in StartPackTankFilling modes
 *
 *  @details:
 *
 */
static VOID dip_manageStartPackTankTransition(DIP_ProcessParam_t * p_ProcessParam)
{
   FLOAT fTankVolume = MES_GetMeasure(MES_TANK_LEVEL_VOL);
   
   if((DIP_MODE_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(((FLOAT)DIST_FIRST_INSTALL_VOLUME <= fTankVolume) ||
         (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_TANK_OVERFLOW)))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
         DO_WRITE(g_DoDistributionSetpoints.IsTheFirstStartUpDist, FALSE);
         p_ProcessParam->pProcessRecord->isFirstStartUp = FALSE;
         SYS_RefreshBackupRecord(SYS_REC_DIST);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the Polisher Pack Status impact on process mode
 *
 *  @details:
 *
 */
static VOID dip_managePolisherStatus(DIP_ProcessParam_t * p_ProcessParam)
{
   CSM_Status_t PolisherStatus = CSM_GetStatus(CSM_POLISHER);

   /* Case of test for manufacturing bench */
   if (CSM_STATUS_TEST == PolisherStatus)
   {
      /* If the Polisher is in Test mode, the process switches to Test mode */
      if((DIP_MODE_TEST != p_ProcessParam->ProcessDescription.ProcessMode)            &&
         (DIP_MODE_TEST_ALARM_STOP != p_ProcessParam->ProcessDescription.ProcessMode) &&
         (DIP_MODE_TEST_RECIRC != p_ProcessParam->ProcessDescription.ProcessMode))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE); /* First we record the mode Standby Old Pack */
         dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
      }
   }
   /* End of manufacturing case */

   /* Case of init */
   if (CSM_STATUS_INIT == PolisherStatus)
   {
      /* If the process is not in INIT mode there is a problem */
      if(DIP_MODE_INITIALIZATION != p_ProcessParam->ProcessDescription.ProcessMode)
      {
         ASSERT(FALSE, "Uncompatible distribution mode for INIT consumable Status");
      }
   }
   /* End of init case */

   if((DIP_MODE_LISTENING == p_ProcessParam->ProcessDescription.ProcessMode)                 ||
      (DIP_MODE_RUNNING == p_ProcessParam->ProcessDescription.ProcessMode)                   ||
      (DIP_MODE_DUMPING == p_ProcessParam->ProcessDescription.ProcessMode)                   ||
      (DIP_MODE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)                ||
      (DIP_MODE_OPTICAP_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (DIP_MODE_OPTICAP_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (DIP_MODE_RESTART_OPTICAP_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)     ||
      (DIP_MODE_TOC_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode)              ||
      (DIP_MODE_TOC_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_NEW_PACK, TRUE);
      }
      else if((CSM_STATUS_TOOL_BYPASS == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_BYPASS, TRUE);
      }
      else
      {
         /* Do nothing */
      }
   }

   if((DIP_MODE_TANK_EMPTYING == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (DIP_MODE_TANK_EMPTYING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_NEW_PACK, TRUE);
      }
   }

   if((DIP_MODE_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_POLISHER_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == PolisherStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_POLISHER_FLUSH, TRUE);
      }
      else if(CSM_STATUS_TOOL_BYPASS == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_POLISHER_FLUSH, TRUE);
      }
      else
      {
         /* Do nothing */
      }
   }
   if((DIP_MODE_CAN_NOT_RESTART_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == PolisherStatus) ||
         (CSM_STATUS_NEW == PolisherStatus) ||
         (CSM_STATUS_OVERRIDDEN == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_POLISHER_FLUSH, TRUE);
      }
   }

   if((DIP_MODE_RESTART_POLISHER_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != PolisherStatus) &&
         (CSM_STATUS_NEW != PolisherStatus) &&
         (CSM_STATUS_OVERRIDDEN != PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_POLISHER_FLUSH, TRUE);
      }
   }

   if((DIP_MODE_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == PolisherStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_STARTPACK_TANK_FILLING, TRUE);
      }
      else if(CSM_STATUS_TOOL_BYPASS == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING, TRUE);
      }
      else
      {
         /* Do nothing */
      }
   }
   if((DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == PolisherStatus) ||
         (CSM_STATUS_NEW == PolisherStatus) ||
         (CSM_STATUS_OVERRIDDEN == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_STARTPACK_TANK_FILLING, TRUE);
      }
   }
   if((DIP_MODE_RESTART_STARTPACK_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != PolisherStatus) &&
         (CSM_STATUS_NEW != PolisherStatus) &&
         (CSM_STATUS_OVERRIDDEN != PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING, TRUE);
      }
   }

   if((DIP_MODE_STANDBY_ALARM == p_ProcessParam->ProcessDescription.ProcessMode)       ||
      (DIP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)    ||
      (DIP_MODE_STANDBY_NEW_PACK == p_ProcessParam->ProcessDescription.ProcessMode)    ||
      (DIP_MODE_STANDBY_BYPASS == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (DIP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_BYPASS == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_BYPASS, TRUE);
      }
      else if(CSM_STATUS_OLD == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
      }
      else if(CSM_STATUS_OVERRIDDEN == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_PACK_OVERRIDE, TRUE);
      }
      else if(CSM_STATUS_NEW == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_NEW_PACK, TRUE);
      }
      else
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_ALARM, TRUE);
      }
   }

   if((DIP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN1_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_BYPASS != PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_SAN1, TRUE);
      }
   }
   if(DIP_MODE_CAN_NOT_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode)
   {
      if(CSM_STATUS_TOOL_BYPASS == PolisherStatus)
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN1, TRUE);
      }
   }
   if((DIP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN2_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_TOOL_BYPASS != PolisherStatus) &&
         (CSM_STATUS_OVERRIDDEN != PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_SAN2, TRUE);
      }
   }
   if(DIP_MODE_CAN_NOT_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode)
   {
      if((CSM_STATUS_TOOL_BYPASS == PolisherStatus) ||
         (CSM_STATUS_OVERRIDDEN == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN2, TRUE);
      }
   }
   if((DIP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN3_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_TOOL_BYPASS != PolisherStatus) &&
         (CSM_STATUS_OVERRIDDEN != PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_SAN3, TRUE);
      }
   }
   if(DIP_MODE_CAN_NOT_RESTART_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode)
   {
      if((CSM_STATUS_TOOL_BYPASS == PolisherStatus) ||
         (CSM_STATUS_OVERRIDDEN == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN3, TRUE);
      }
   }
   if((DIP_MODE_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN4_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_TOOL_BYPASS != PolisherStatus) &&
         (CSM_STATUS_OVERRIDDEN != PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_SAN4, TRUE);
      }
   }
   if(DIP_MODE_CAN_NOT_RESTART_SAN4 == p_ProcessParam->ProcessDescription.ProcessMode)
   {
      if((CSM_STATUS_TOOL_BYPASS == PolisherStatus) ||
         (CSM_STATUS_OVERRIDDEN == PolisherStatus))
      {
         dip_setNewMode(p_ProcessParam, DIP_MODE_RESTART_SAN4, TRUE);
      }
   }
}

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

