/*!
* @file dip_task.c
*
* @brief Distribution Task
*
* @details Contains Distribution stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: dip_task.c 539 2014-08-2#$: Filename
* $Rev:: 539                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-08-21 10:13:24 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_task.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/
#define  DELAY500MS     ((UINT32)500)


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "msg_type.h"
#include "dip_rec.h"
#include "sys_softerr.h"
#include "dip_data.h"
#include "mes_api.h"
#include "sys_api.h"
#include "sys_log.h"
#include "dip_private.h"
#include "do_itf.h"
#include "do_def.h"

/*!
 *  @defgroup Distribution Distribution module
 *
 *  @brief Distribution task module
 *
 *  @details Distribution task is in charge of :
 *       @li Water distribution process
 *
 *  @{
 */


/* Declarations --------------------------------------------------------------*/

OsHandle_t hDipMboxHld;                   /*!< Distribution task mailbox */

static OsHandle_t hDipSemaList[ 3u ];   /*!< Distributionsemaphores handle array */
/* Defines -------------------------------------------------------------------*/

#define DIP_MSG_SEM_IDX ((INT32)0)
#define DIP_TIM_SEM_IDX ((INT32)1)

/* Code ----------------------------------------------------------------------*/


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Initialized Distribution data
 *
 *  @details  Ensure all distribution data are initialized before Measure task start
 *
 */
static VOID InitData(VOID)
{
   eOsRc_t rc;
   static CONST PLAINCHAR* MBoxName = "DIP_MBX";

   static CONST PLAINCHAR* SemaNameArray [3u] =
   {
      "DIP_MSG_SEM",
      "DIP_TIM_SEM",
      NULL
   };

   /*! resolve seampahore ID to name  */
   rc=OS_SemaLookUpL(SemaNameArray, hDipSemaList );
   ASSERT( OS_GOOD == rc ,"Wrong OS_SemaLookUpL call");
   /*! resolve Mailbox  ID to name  */
   rc= OS_MboxLookUp( MBoxName, &hDipMboxHld );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxLookUpcall");

   /*! Link MailBox to Semaphore    */
   rc=OS_MboxDefSema(MBoxName, SemaNameArray[DIP_MSG_SEM_IDX] );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxDefSema call ");

   dip_readCanOpenSetpoints();

   /*! Initialize Distribution Process Parameters to force the Initialization */
   DistributionProcessParameters.ProcessDescription.ProcessMode = LAST_DIP_MODE;
   DistributionProcessParameters.ProcessDescription.ProcessSequence = LAST_DIP_SEQUENCE;
   DistributionProcessParameters.ProcessDescription.ProcessState = LAST_DIP_STATE;
   DistributionProcessParameters.TransitionDescription.TransitionType = LAST_DIP_TRANSITION;
   dip_setNewMode(&DistributionProcessParameters, DIP_MODE_INITIALIZATION, FALSE);

   /*! Get the Recorded Parameters in Flash */
   DistributionProcessParameters.pProcessRecord = (Dip_Record_t *)SYS_RestoreRecord (SYS_REC_DIST);
   
   DO_WRITE(g_DoDistributionSetpoints.IsTheFirstStartUpDist, DistributionProcessParameters.pProcessRecord->isFirstStartUp);
}


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Distribution task
 *
 *  @details   TBC
 *
 */
ATTRIBUTE_TASK VOID distributionTask(VOID)
{
   INT32       sdwSemaListOffset;
   DIP_Msg_t   dipMsg;

   SYSLOG(INFO, "Distribution task .......: START");

   InitData();

   g_bAppTaskInit++;

   while ( FALSE == g_doLaunchApp )
   {
      OS_TaskDelay(DELAY500MS);
   }

   while (g_isDistributionTaskActive)
   {
      sdwSemaListOffset = OS_SemaWaitLHI( (OsHandle_t*)hDipSemaList ) ;

      switch ( sdwSemaListOffset )
      {
         case DIP_MSG_SEM_IDX:
         {
            /*! Just get the message (does not wait for it) */
            MSG_GetWaitH( hDipMboxHld, (psMSG_t)&dipMsg, (UINT8)0U /* Get */ );
            /*! Analyses message content */
            DIP_MsgParser(&dipMsg);
            break;
         }

         case DIP_TIM_SEM_IDX:
         {
            dip_distributionStateMachine(DIP_LAST_ORDER);
            break;
         }

         default:
         {
            ASSERT( FALSE,"Wrong message ID receive in AlertAndAlarm task");
            break;
         }
      }

   }

   SYSLOG(INFO, "Distribution task .......: END");

   OS_TaskTerminate(OS_SELFTASK);
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

