/*!
 * @file mes_can.c
 * @author Alain Keraudy
 * @date $Date: 2014-10-15 16:16:32 +0200 (mer. 15 oct. 2014) $
 *
 * @brief Measure CAN interface
 *
 * @details CAN interface to allow sending PDO
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Rev:: 601                   $: Revision of last commit
 * $Author:: m175711            $: Author of last commit
 * $Date:: 2014-10-15 16:16:32 #$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_can.c $: URL of this file
 * @endcond
 *
 */

/* Includes ------------------------------------------------------------------*/
#include "CanOpenIface.h"
#include "mes_data.h"
#include "mes_can.h"
#include "do_itf.h"

/*!
 *
 * @defgroup Measure_CAN_Interface CAN interface
 *
 * @ingroup Measure
 *
 * @brief Measure CAN interface description
 *
 * @{
 */

/* Definitions --------------------------------------------------------------*/
#define BIGGEST_PDO_TO_SEND   (20U)

/* Declarations --------------------------------------------------------------*/

BOOL g_TpdoReadyToStart[BIGGEST_PDO_TO_SEND];

/* Code ----------------------------------------------------------------------*/
#ifdef __CANOPEN_ITF_BY_ID__
/*!
 *  @brief retrieve TPDO from OjbId given
 *
 *  @details TBC
 *
 */
UINT8 MES_CanGetTPDO(UINT32 dwObjIDSearch)
{
   BOOL isFound = FALSE;
   INT32 doRetStatus;
   UINT8 bPdo = 0;

   for (UINT32 dwIdx = 0x1A00; dwIdx < 0x1A20; dwIdx++)
   {
      UINT8 bLastSubIdx;
      UINT32 dwObjID = (dwIdx << 16) | (1 << 3);

      // Get _lastSubIdx in 0x1A0x0000 dwObjID
      doRetStatus = DictObjGetRecord(dwObjID, &bLastSubIdx);
      if (doRetStatus < 0)
      { continue; }
      //set size to U32
      dwObjID &= ~((UINT32)0x0FFU); //set no size
      dwObjID |= ((UINT32)4 << 3); //set size to U32
      for (UINT32 dwSubIdx = 1; dwSubIdx <= bLastSubIdx; dwSubIdx++)
      {
         UINT32 dwObjIDCarried;
         // Get 0x1A0x000y dwObjID
         doRetStatus = DictObjGetRecord((dwObjID | (dwSubIdx << 8)), &dwObjIDCarried);
         if (dwObjIDSearch == dwObjIDCarried)
         {
            isFound = TRUE;
            break;
         }
      }
      if (TRUE == isFound)
      {
         break;
      }
      bPdo++;
   }

   if (TRUE == isFound)
   {
      return bPdo;
   }
   else
   {
      return MES_CAN_TPDO_NOT_FOUND;
   }
}
#endif // __CANOPEN_ITF_BY_ID__


/*!
 *  @return None
 *
 *  @brief  Handle Record sending every seconds with Can PDO msg TX
 */
VOID MES_CanSendRecords(VOID)
{
   UINT16 tpdo;
   for (tpdo = 0U; tpdo < (UINT16)BIGGEST_PDO_TO_SEND; tpdo++)
   {
      if (TRUE == g_TpdoReadyToStart[tpdo])
      {
         canSendPdo(tpdo);
         g_TpdoReadyToStart[tpdo] = FALSE;
      }
   }
}


/*!
 *  @param  bTpdo :           IN    UINT8                 <br>
 *       transmit Pdo id
 *
 *  @return None
 *
 *  @brief  Handle Record Update once measure has been performed
 *
 */
VOID MES_CanRecordUpdate(UINT8 bTpdo)
{
   if( (bTpdo != 0U) && (bTpdo < (UINT8)BIGGEST_PDO_TO_SEND) )
   {
      g_TpdoReadyToStart[bTpdo] = TRUE;
   }
}


/*!
 * @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

