/*!
* @file mes_hardware.c
*
* @brief Hardware monitoring
*
* @details Contains Hardware monitoring procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_hardware.c 454 201#$: Filename
* $Rev:: 454                   $: Revision of last commit
* $Author:: x157168            $: Author of last commit
* $Date:: 2014-06-06 15:00:38 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_hardware.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_hw.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "system_parameters.h"
#include "measure.h"



/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup HardwareMon_Measure Hardware monitor
 *  @ingroup Measure
 *  @brief Hardware monitoring acquistion procedures
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#define SUPPLY_VOLTAGE_5V_THRESHOLD    (4.80f)
#define SUPPLY_VOLTAGE_5V_BIT          (DIL_HW_VREF+1)


/* Declarations --------------------------------------------------------------*/
#ifndef  D_BSP_STM3240GEVAL
static DIL_HANDLE dilHwHdl = NULL; /*!< Harware  DIL handle */

/* Codes ---------------------------------------------------------------------*/

/*!
 *  @return None
 *
 *  @brief Log Hw monitor
 *
 *  @details NA
 */
static  VOID LogHwMonitoringStatus (UINT8 bHwStatus, UINT8 bTestBit,  PLAINCHAR* AlarmString, PLAINCHAR*  NormalString )
{
   if(bHwStatus & ( (UINT8)1<<bTestBit))
   {
      SYSLOG(INFO, AlarmString);
   }
   else
   {
      SYSLOG(INFO, NormalString);
   }
}
#endif //D_BSP_STM3240GEVAL

/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used.
 *
 *  @return void
 *
 *  @brief Monitor Hardware 1000 ms
 *
 *  @details
 *     -# First , record value
 *     -# if values change below theshold , Log alarm
 */
VOID MES_HardwareMonitor(BOOL isSecondBoundary)
{
#ifndef  D_BSP_STM3240GEVAL
   static UINT8 bHwAlarmStatus=0u;

   DIL_RC   ret;
   DIL_UINT32 dwHwStatus = 0U;
   DIL_FLOAT f5VMeasure = 0.0F;

   ret = dil_ioctldevice(dilHwHdl, (DIL_UINT32)IOCTL_CMD_HW_MONITORING_GET_STATUS ,
                         (DIL_LPVOID)&dwHwStatus, sizeof(dwHwStatus), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL HW Get status in call");

   ret = dil_ioctldevice(dilHwHdl, (DIL_UINT32)IOCTL_CMD_HW_MONITORING_GET_5V ,
                         (DIL_LPVOID)&f5VMeasure, sizeof(f5VMeasure), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL HW Get 5V Pwr supply status in call");

   if(f5VMeasure<SUPPLY_VOLTAGE_5V_THRESHOLD)
   {
      dwHwStatus|=SUPPLY_VOLTAGE_5V_BIT;
   }

   if((UINT8)dwHwStatus != bHwAlarmStatus )
   {
      LogHwMonitoringStatus((UINT8)dwHwStatus,DIL_HW_TENSION5V,   "Alim   5V : ALARM","Alim   5V : OK");
      LogHwMonitoringStatus((UINT8)dwHwStatus,DIL_HW_TENSION18V,  "Alim  18V : ALARM","Alim  18V : OK");
      LogHwMonitoringStatus((UINT8)dwHwStatus,DIL_HW_TENSION24V,  "Alim  24V : ALARM","Alim  24V : OK");
      LogHwMonitoringStatus((UINT8)dwHwStatus,DIL_HW_TEMP1,       "Int temp  : ALARM","Int temp  : OK");
      LogHwMonitoringStatus((UINT8)dwHwStatus,DIL_HW_VREF,        "Alim VRef : ALARM","Alim VRef : OK");
      SYSLOG(INFO,                                          "Alim      : %.02fV", f5VMeasure);
      bHwAlarmStatus=(UINT8)dwHwStatus;
   }

   /* Update CAN objects */
   DO_WRITE(g_DoPowerSupplyDiagnostic.PowerSupplyStatus,(UINT8)dwHwStatus);
   DO_WRITE(g_DoPowerSupplyDiagnostic.Voltage5V,f5VMeasure);
#endif

   NOT_USED_PARAM(isSecondBoundary);
}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details
 */
VOID MES_InitHardwareMonitoringDriver(VOID)
{
#ifndef  D_BSP_STM3240GEVAL
   DIL_RC   ret;

   /* DIL feed pressure init */
   ret = dil_opendevice(DIL_NAME_HARDWARE_MONITORING, &dilHwHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL HardwareMon in init");


#endif
}


/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

