/*!
* @file rop_sequence.c
*
* @brief Production Task
*
* @details Contain all Production stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: rop_sequence.c 689 2014-12-24 09:#$: Filename
* $Rev:: 689                              $: Revision of last commit
* $Author:: m175711                       $: Author of last commit
* $Date:: 2014-12-24 10:08:18 +0100 (mer.#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/production/src/rop_sequence.c $: URL of this file
* @endcond
*
*/

/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @ingroup RopStateMachine
 *  @{
 */

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "rop_api.h"
#include "rop_rec.h"
#include "sys_softerr.h"
#include "rop_data.h"
#include "mes_api.h"
#include "sys_api.h"
#include "sys_log.h"
#include "rop_private.h"
#include "system_parameters.h"
#include "dip_api.h"
#include "mvm_api.h"
#include "do_itf.h"
#include "do_def.h"
#include "act_api.h"

/* Defines -------------------------------------------------------------------*/
#define ROP_FLUSH_FLOWRATE_BY_SECOND (111U) // (400 L/H in mL by sec = 400 * 1000 / 3600) Use to compute the remaining time of sequence with Flush and Short Flush
#define ROP_LIMITED_FLUSH_FLOWRATE_BY_SECOND (28U) // (100 L/H in mL by sec = 100 * 1000 / 3600) Use to compute the remaining time of sequence with Flush and Short Flush
#define ROP_MAX_TANK_FILLING_DELAY (259200U) // 3 days in seconds
#define ROP_RECORD_LAST_TANK_FILLING_FREQUENCY (60000U) // 1 minutes in msec
#define ROP_ACTUATORS_TEST_FREQUENCY (300000U) // 5 minutes in msec
#define ROP_TIMER_RECORD_FREQUENCY (20000U) // 20 secondes in msec
#define ROP_VOLUME_COUNTER_DELAY (2000U) // 2 secondes in msec
#define ROP_TEST_TAP_PRESSURE_DELAY (4000U) // 4 secondes in msec
#define ROP_SLOW_TRANSITION_DURATION (420U) // 7 minutes in sec
#define ROP_TEST_RO_PUMP_DURATION (60U) // 1 minute in sec
#define ROP_HOUR_TO_MS (3600000) // 1 hour in msec

/* Declarations --------------------------------------------------------------*/
typedef enum
{
   RO_POLYPHOSPHATE_REMOVAL_DURATION,
   PROGARD_FLUSH_DURATION,
   PROGARD_CL2_FLUSH_VOLUME,
   RO_CL2_FLUSH_VOLUME,
   RO_PH_FLUSH_VOLUME
} SystemDependentParameters_e;

typedef struct rop_Rinsing_Records_s
{
   UINT16 rinsingDurationMeasure;
   BOOL rinsingDurationRecord;
   FLOAT rinsingPressure;
} rop_Rinsing_Records_t;

typedef struct rop_StartSanCheck_s
{
   BOOL isDistPressureOK;
   TimeCounter_t startSanCounter;
} rop_StartSanCheck_t;

static DOUBLE DrainCounterRef;
static DOUBLE PermeateCounterRef;
static BOOL CheckEnable[CHECK_MAX];
static BOOL VolumeCounterEnable;
static BOOL FirstFlushHasBeenDone = FALSE;
/* CCR 361
static BOOL FirstUfWaitHasBeenDone = FALSE;
*/
static BOOL tapPressureMeasureEnable = FALSE;
static rop_Rinsing_Records_t RinsingRecords;
static TimeCounter_t rop_TemporaryStateTimeCounter;

static VOID rop_manageSequenceInitialization(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceStandby(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceWaiting(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceTankFilling(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceRoDiagnostic(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceAlarmStop(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequencePause(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceProgardFlush(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceFirstInstall(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceRoRinse(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceCl2Cleaning(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequencePhCleaning(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceReagentRemoval(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceTest(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceTestRinse(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageSequenceTestFlush(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_setNewState(ROP_ProcessParam_t * p_ProcessParam, RopState_e NewState);

static BOOL isStartTankFillingConditions (VOID);
static BOOL isStopTankFillingConditions (VOID);
static BOOL isRinsingEndConditions (UINT32 rinsingStateTimeCounter);
static UINT32 rop_getSystemDependentParameters(SystemDependentParameters_e SystemDependentParameters);
static BOOL isStartUfBackwashConditions (VOID);
static BOOL isStartFlushConditions (VOID);
static BOOL isStartPreFlushConditions (VOID);
static VOID rop_StartFeedVolumeCounter (VOID);
static UINT32 rop_getFeedVolumeCounter (VOID);
static VOID rop_stopSpecialCounters (VOID);
static VOID rop_manageStartMeasureAndRecord (ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageStopMeasureAndRecord (RopState_e newState);
static VOID rop_manageDistributionAutoDumpStart (VOID);
static VOID rop_manageDistributionAutoDumpStop (VOID);
static VOID rop_manageTestActuators (UINT32 Timer);
static VOID rop_manageManualRegulation(ROP_ProcessParam_t * p_ProcessParam);

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the different type of sequence to call
 *
 *  @details:
 *
 */
VOID rop_manageSequence (ROP_ProcessParam_t * p_ProcessParam)
{
   BOOL ReEnterSequence = TRUE;

   while (TRUE == ReEnterSequence)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessSequence)
      {
         case ROP_SEQUENCE_INITIALIZATION:   {rop_manageSequenceInitialization(p_ProcessParam);  break;}
         case ROP_SEQUENCE_STANDBY:          {rop_manageSequenceStandby(p_ProcessParam);         break;}
         case ROP_SEQUENCE_WAITING:          {rop_manageSequenceWaiting(p_ProcessParam);         break;}
         case ROP_SEQUENCE_TANK_FILLING:     {rop_manageSequenceTankFilling(p_ProcessParam);     break;}
         case ROP_SEQUENCE_RO_DIAGNOSTIC:    {rop_manageSequenceRoDiagnostic(p_ProcessParam);    break;}
         case ROP_SEQUENCE_ALARM_STOP:       {rop_manageSequenceAlarmStop(p_ProcessParam);       break;}
         case ROP_SEQUENCE_PAUSE:            {rop_manageSequencePause(p_ProcessParam);           break;}
         case ROP_SEQUENCE_FIRST_INSTALL:    {rop_manageSequenceFirstInstall(p_ProcessParam);    break;}
         case ROP_SEQUENCE_PROGARD_FLUSH:    {rop_manageSequenceProgardFlush(p_ProcessParam);    break;}
         case ROP_SEQUENCE_RO_RINSE:         {rop_manageSequenceRoRinse(p_ProcessParam);         break;}
         case ROP_SEQUENCE_PH_CLEANING:      {rop_manageSequencePhCleaning(p_ProcessParam);      break;}
         case ROP_SEQUENCE_CL2_CLEANING:     {rop_manageSequenceCl2Cleaning(p_ProcessParam);     break;}
         case ROP_SEQUENCE_REAGENT_REMOVAL:  {rop_manageSequenceReagentRemoval(p_ProcessParam);  break;}
         case ROP_SEQUENCE_TEST:             {rop_manageSequenceTest(p_ProcessParam);            break;}
         case ROP_SEQUENCE_TEST_RINSE:       {rop_manageSequenceTestRinse(p_ProcessParam);       break;}
         case ROP_SEQUENCE_TEST_FLUSH:       {rop_manageSequenceTestFlush(p_ProcessParam);       break;}
         default:
         {
            ERROR("Unknown Process Seqeunce: %d", p_ProcessParam->ProcessDescription.ProcessSequence);
            break;
         }
      }
      ReEnterSequence = p_ProcessParam->ProcessDescription.isEnteringSequence;
   }

   // Manage the Auto-Dump Flag for distribution process
   rop_manageDistributionAutoDumpStop();

   // Manage the records and check for alarms
   rop_manageStartMeasureAndRecord(p_ProcessParam);
   
   // Manage the manual regulation case
   rop_manageManualRegulation(p_ProcessParam);
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewState [OUT] RopState_e <br>
 *                    New Process State
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Set a new state
 *
 *  @details: If the new state is different from the old one, the parameters are reset
 *            and if the associated transition is different, the process re-enter a new transition
 *
 */
static VOID rop_setNewState(ROP_ProcessParam_t * p_ProcessParam, RopState_e NewState)
{
   RopTransition_e rop_transition;

   ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter();

   if (NewState != p_ProcessParam->ProcessDescription.ProcessState)
   {
      p_ProcessParam->ProcessDescription.ProcessState = NewState;
      rop_transition = rop_getAssociatedTransition(NewState);
      if (rop_transition != p_ProcessParam->TransitionDescription.TransitionType)
      {
         p_ProcessParam->TransitionDescription.TransitionType = rop_transition;
         p_ProcessParam->TransitionDescription.TransitionStatus = ROP_TRANSITION_START;
         rop_manageTransition(&(p_ProcessParam->TransitionDescription));
      }
      
      rop_manageStopMeasureAndRecord (NewState);
   }
}

/*!
 *  @return BOOL
 *  @retval TRUE: if an UF Backwash is required
 *  @retval FASLE: if no UF Backwash is required
 *
 *  @brief  Allow the process to know if a backwash is required
 *
 *
 */
static BOOL isStartUfBackwashConditions (VOID)
{
   BOOL StartUfBackwash = FALSE;

   /* CCR 361
   if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_UF)) // if UF is activated
   {
      if (FALSE == FirstUfWaitHasBeenDone)
      {
         StartUfBackwash = TRUE;
      }
      else if ((((UINT32)ProductionProcessSetpoints.ExtUfBackFrequencySP) * 3600U * 1000U) <= TMC_GetTimeCounter(ProductionProcessParameters.ProcessDescription.LastUfBackwashTimeCounter))
      {
         StartUfBackwash = TRUE;
      }
      else
      {
         // Do Nothing
         StartUfBackwash = FALSE;
      }
   }
   */
   
   return(StartUfBackwash);
}

/*!
 *  @return BOOL
 *  @retval TRUE: if a flush is required
 *  @retval FASLE: if no flush is required
 *
 *  @brief  Allow the process to know if a flush is required
 *
 *
 */
static BOOL isStartFlushConditions (VOID)
{
   BOOL startFlush = FALSE;

   if (FALSE == FirstFlushHasBeenDone)
   {
      startFlush = TRUE;
   }
   else
   {
      startFlush = ((((UINT32)ProductionProcessSetpoints.AutoFlushFrequencySP) * 3600U * 1000U) <= TMC_GetTimeCounter(ProductionProcessParameters.ProcessDescription.LastFlushTimeCounter));
   }
   return (startFlush);
}

/*!
 *  @return BOOL
 *  @retval TRUE: if a pre-flush is required
 *  @retval FASLE: if no pre-flush is required
 *
 *  @brief  Allow the process to know if a pre-flush is required
 *
 *
 */
static BOOL isStartPreFlushConditions (VOID)
{
   BOOL startFlush = FALSE;

   if (FALSE == FirstFlushHasBeenDone)
   {
      startFlush = TRUE;
   }
   else
   {
      startFlush = ((((UINT32)ProductionProcessSetpoints.PreFillingFrequencySP) * 3600U * 1000U) <= TMC_GetTimeCounter(ProductionProcessParameters.ProcessDescription.LastFlushTimeCounter));
   }
   return (startFlush);
}

/*!
 *  @return BOOL
 *  @retval TRUE: if a Progard get Chlore
 *  @retval FASLE: if no Progard get Chlore
 *
 *  @brief  Allow the process to know if a Progard get Chlore inside
 *
 *
 */
BOOL rop_isProgardWithChlore (VOID)
{
   BOOL ProgardWithChlore = FALSE;

   if (CSM_PG_WITH_CL == CSM_GetType(CSM_PROGARD1))
   {
      ProgardWithChlore = TRUE;
   }
   else if ((SYSTEM_TWO_PROGARDS == SYS_GetNbProgards()) &&
            (CSM_PG_WITH_CL == CSM_GetType(CSM_PROGARD2)))
   {
      ProgardWithChlore = TRUE;
   }
   else
   {
      ProgardWithChlore = FALSE;
   }
   return (ProgardWithChlore);
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Reset the Flush Timer
 *
 */
VOID rop_resetFlushTimer (ROP_ProcessParam_t * p_ProcessParam)
{
   p_ProcessParam->ProcessDescription.LastFlushTimeCounter = TMC_StartTimeCounter();
   FirstFlushHasBeenDone = TRUE;
}

/*!
 *  @return BOOL
 *  @retval TRUE: if a Tank Filling is required
 *  @retval FASLE: if no Tank Filling is required
 *
 *  @brief  Allow the process to know if a Tank Filling is required
 *
 */
static BOOL isStartTankFillingConditions (VOID)
{
   BOOL StartTankFilling = FALSE;
   if((((FLOAT)ProductionProcessSetpoints.TankRefillSP) >= MES_GetMeasure(MES_TANK_LEVEL_PC)) &&
      (FALSE == DIP_isProductionDisable()))
   {
      StartTankFilling = TRUE;
   }

   return(StartTankFilling);
}

/*!
 *  @return BOOL
 *  @retval TRUE: if the Tank Filling needs to be stopped
 *  @retval FASLE: if the Tank Filling is not done
 *
 *  @brief  Allow the process to know if the Tank Filling needs to be stopped
 *
 */
static BOOL isStopTankFillingConditions (VOID)
{
   BOOL StopTankFilling = FALSE;
   UINT32 ActualStateTimer = TMC_GetTimeCounter(ProductionProcessParameters.ProcessDescription.StateTimeCounter);
   FLOAT TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);

   if((FLOAT)STOP_TANK_FILLING_TANK_LEVEL <= TankLevelMm) // Mesure en mm
   {
      StopTankFilling = TRUE;
   }

   /* CCR 361
   if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_UF)) // if UF is activated
   {
      if ((((UINT32)ProductionProcessSetpoints.ExtUfBackFrequencySP) * 3600U * 1000U) <= ActualStateTimer)
      {
         StopTankFilling = TRUE;
      }
   }
   */

   if ((((UINT32)ProductionProcessSetpoints.TankFillingAutoStopDurationSP) * 3600U * 1000U) <= ActualStateTimer)
   {
      StopTankFilling = TRUE;
   }

   if (TRUE == DIP_isProductionDisable())
   {
      StopTankFilling = TRUE;
   }

   return(StopTankFilling);
}

/*!
 *  @return BOOL
 *  @retval TRUE: if the Rinsing is done
 *  @retval FASLE: if the Rinsing is not done
 *
 *  @brief  Allow the process to know if the Rinsing is done
 *
 */
static BOOL isRinsingEndConditions (UINT32 rinsingStateTimeCounter)
{
   BOOL isRinsingOK = FALSE;
   FLOAT RoRejection = MES_GetMeasure(MES_RO_REJECTION);
   FLOAT FceMeasure = MES_GetMeasure(MES_FCE);

   if ((((UINT32)RINSE_DURATION_MAX + 1U) * 1000U) <= rinsingStateTimeCounter) /* +1 allows to let the alarm FCE > SP to be set */
   {
      // rinsing has last for more than RINSE_DURATION_MAX
      isRinsingOK = TRUE;
   }
   else if ((((UINT32)RINSE_DURATION_MIN + 1U) * 1000U) <= rinsingStateTimeCounter) /* +1 allows to let the check alarm start */
   {
      /*
       * rinsing has last for more than RINSE_DURATION_MIN
       * check water quality
       */
      if ((RoRejection >= (ProductionProcessParameters.ProcessDescription.LastRoRejection * 0.99F)) &&
          (FceMeasure <= (ALARM_FCE_MAX * 0.90F)))
      {
         isRinsingOK = TRUE;
      }
   }
   else
   {
      isRinsingOK = FALSE;
   }
   return (isRinsingOK);
}

/*!
 *  @return none
 *
 *  @brief  Manage the Distribution Auto-Dump Stop
 *
 *  @details: If the auto-dump is started, the RO process checks the tank level to stop the dump
 */
static VOID rop_manageDistributionAutoDumpStop (VOID)
{
   FLOAT TankLevelPercent = MES_GetMeasure(MES_TANK_LEVEL_PC);
   BOOL autoDumpFlag = DIP_GetAutoDumpFlag();

   if ( TRUE == autoDumpFlag)
   {
      if(((FLOAT)ProductionProcessSetpoints.TankRefillSP - (FLOAT)DIST_STOP_AUTO_DUMP_TANK_LEVEL_GAP) >= TankLevelPercent)
      {
         // Disable the Auto-Dump
         DIP_SetAutoDumpFlag(FALSE);
      }
   }
}

/*!
 *  @return none
 *
 *  @brief  Manage the Distribution Auto-Dump Start
 *
 *  @details: Set the auto-dump flag to TRUE when there is no Tank Filling for a long time
 */
static VOID rop_manageDistributionAutoDumpStart (VOID)
{
   time_t systime;
   FLOAT TankLevelPercent = MES_GetMeasure(MES_TANK_LEVEL_PC);
   BOOL autoDumpFlag = DIP_GetAutoDumpFlag();

   if (FALSE == autoDumpFlag)
   {
      if ((ProductionProcessParameters.pProcessRecord->LastTankFillingTime > 0U) &&
          (TankLevelPercent > ((FLOAT)ProductionProcessSetpoints.TankRefillSP)))
      {
         // Get the system date.
         DO_READ(systime, g_DoDate);

         if ((systime > ProductionProcessParameters.pProcessRecord->LastTankFillingTime) &&
             ((systime - ProductionProcessParameters.pProcessRecord->LastTankFillingTime) > ROP_MAX_TANK_FILLING_DELAY)) // Last Tank Filling was in more than 3 days
         {
            // Enable the Auto-Dump
            DIP_SetAutoDumpFlag(TRUE);
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceInitialization(ROP_ProcessParam_t * p_ProcessParam)
{
   CSM_Status_t Progard1Status = CSM_STATUS_MAX;
   CSM_Status_t Progard2Status = CSM_STATUS_MAX;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set in WAIT state
      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;

      // Motor Valve initialization
      MVM_StartInitialization();
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      if ((MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION)))
      {
         /* Initialize flow measure counter */
         MES_FlowCounterStart(MES_PERMEATE_FLOWRATE);
         MES_FlowCounterStart(MES_DRAIN_FLOWRATE);
         MES_FlowCounterStart(MES_EDI_PRODUCTION_FLOWRATE);
         
         Progard1Status = CSM_GetStatus(CSM_PROGARD1);
         if (SYSTEM_ONE_PROGARD != SYS_GetNbProgards())
         {
            Progard2Status = CSM_GetStatus(CSM_PROGARD2);
         }
         
         if((CSM_STATUS_INIT != Progard1Status) &&
            (CSM_STATUS_INIT != Progard2Status))
         {
            if(TRUE == p_ProcessParam->pProcessRecord->isFirstStartUp)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL, TRUE);
            }
            else
            {
               // Recover the interrupted sequence if it is coherent
               if ((ROP_MODE_INITIALIZATION != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (ROP_MODE_TEST != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (ROP_MODE_TEST_ALARM_STOP != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (ROP_MODE_TEST_PAUSE != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (ROP_MODE_TEST_RINSE != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (ROP_MODE_TEST_FLUSH != p_ProcessParam->pProcessRecord->InterruptedProcessMode))
               {
                  rop_setNewMode(p_ProcessParam, p_ProcessParam->pProcessRecord->InterruptedProcessMode, FALSE);
               }
               else
               {
                  // else force to Standby
                  rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
               }
            }
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceStandby(ROP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      rop_stopSpecialCounters();
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      if(TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable)
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceWaiting(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      rop_stopSpecialCounters();
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_WAIT:
         {
            // Manage the Auto-Dump Flag for distribution process
            rop_manageDistributionAutoDumpStart();

            if(TRUE == isStartFlushConditions()) // Last Flush was more than AutoFlushFrequencySP
            {
               // Switch to SHORT FLUSH MESURE
               rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
            }
            else if (TRUE == ProductionProcessSetpoints.isAutoDiagRequested)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_DAILY_RO_DIAGNOSTIC, TRUE);
            }
            else if (TRUE == isStartTankFillingConditions())
            {
               // Set the Distribution Auto-Dump to FALSE
               DIP_SetAutoDumpFlag(FALSE);
               // Start the tank filling
               rop_setNewMode(p_ProcessParam, ROP_MODE_TANK_FILLING, TRUE);
            }
            else
            {
               /* Do Nothing */
            }
            break;
         }

         case ROP_STATE_SHORT_FLUSH_MEASURE:
         {
            stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

            if(rop_getSystemDependentParameters(RO_POLYPHOSPHATE_REMOVAL_DURATION) <= stateTimer) // After RO_POLYPHOSPHATE_REMOVAL_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Switch to WAIT
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Waiting", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceTankFilling(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT ediDeltaPressure;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), ProductionProcessSetpoints.RoRecoverySP);

      // Set tank level saturation
      MES_TankLevelSaturation(TRUE);

      if(TRUE == isStartUfBackwashConditions()) // Last UF Backwash was more than ExtUfBackwashFrequency
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_UF_WAIT);
      }
      else if(TRUE == isStartPreFlushConditions()) // Last Flush was more than PreFillingFrequencySP
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
         p_ProcessParam->ProcessDescription.StateNumber = 1U;
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
         p_ProcessParam->ProcessDescription.StateNumber = 2U;
      }
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      DO_READ(ProductionProcessParameters.pProcessRecord->LastTankFillingTime, g_DoDate); // Record the start time of the tank filling
      SYS_RefreshBackupRecord(SYS_REC_PROD);
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_UF_WAIT:
         {
            if((2000U <= stateTimer) && // After 2 second
               (3000U >= stateTimer) )
            {
                // Switch OFF external UF
                ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);
            }

            if((((UINT32)ProductionProcessSetpoints.ExtUfBackDurationSP) * 60U * 1000U) <= stateTimer) // After ExtUfBackDurationSP
            {
               // Reset Backwash Timer
               p_ProcessParam->ProcessDescription.LastUfBackwashTimeCounter = TMC_StartTimeCounter();

               /* CCR 361
               FirstUfWaitHasBeenDone = TRUE;
               */

               if(TRUE == isStartPreFlushConditions()) // Last Flush was more than PreFillingFrequencySP
               {
                  rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
                  p_ProcessParam->ProcessDescription.StateNumber = 1U;
               }
               else
               {
                  rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
                  p_ProcessParam->ProcessDescription.StateNumber = 2U;
               }
            }
            break;
         }

         case ROP_STATE_SHORT_FLUSH_MEASURE:
         {
            if(1U == p_ProcessParam->ProcessDescription.StateNumber)
            {
               if(rop_getSystemDependentParameters(RO_POLYPHOSPHATE_REMOVAL_DURATION) <= stateTimer) // After RO_POLYPHOSPHATE_REMOVAL_DURATION
               {
                  // Reset Flush Timer
                  rop_resetFlushTimer(p_ProcessParam);
                  rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
                  p_ProcessParam->ProcessDescription.StateNumber = 2U;
               }
            }
            else
            {
               if(((UINT32)PRE_RINSE_FLUSH_DURATION * 1000U) <= stateTimer) // After PRE_RINSE_FLUSH_DURATION
               {
                  // Switch to RINSE
                  rop_setNewState(p_ProcessParam, ROP_STATE_RINSE_MESURE);
                  p_ProcessParam->ProcessDescription.StateNumber = 0U;
               }
            }
            break;
         }

         case ROP_STATE_RINSE_MESURE:
         {
            if(TRUE == isRinsingEndConditions(stateTimer))
            {
               // Record Rinsing Duration
               RinsingRecords.rinsingDurationMeasure = (UINT16)(stateTimer/1000U);
               RinsingRecords.rinsingDurationRecord = TRUE;
               RinsingRecords.rinsingPressure = MES_GetMeasure(MES_RO_PRESSURE);

               // Switch to TANK FILLING
               rop_setNewState(p_ProcessParam, ROP_STATE_TANK_FILLING_MESURE);
            }
            else if (TRUE == DIP_isProductionDisable())
            {
               // Sequence is finised, switch to WAITING mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
            }
            break;
         }

         case ROP_STATE_TANK_FILLING_MESURE:
         {
            // Reset Flush Timer
            rop_resetFlushTimer(p_ProcessParam);
               
            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY * 1000U) <= stateTimer) // Timer ok for mesure
            {
               ProductionProcessParameters.ProcessDescription.LastRoRejection = MES_GetMeasure(MES_RO_REJECTION);
            }

            if ((TRUE == RinsingRecords.rinsingDurationRecord) &&
                (((UINT32)START_REGULATION_MEASURE_DELAY * 1000U) <= stateTimer)) // After regulation delay
            {
               RinsingRecords.rinsingDurationRecord = FALSE;
               ediDeltaPressure = (MES_GetMeasure(MES_RO_PRESSURE) - RinsingRecords.rinsingPressure);
               if (ediDeltaPressure < 0.0F)
               {
                  ediDeltaPressure=0.0F;
               }
               else if (ediDeltaPressure > 65.534F)
               {
                  ediDeltaPressure=65.534F; /* Value in mBar limited to UINT16, See under */
               }
               else
               {
                  /* Do Nothing */
               }

               MES_SetRoBackPressure(MES_RO_PERMEABILITY,ediDeltaPressure);
               ediDeltaPressure = ((ediDeltaPressure * 1000.0F) + 0.5f); // Value in mbars for PDO
               rop_sendRinsingDurationPDO (RinsingRecords.rinsingDurationMeasure, (UINT16)ediDeltaPressure);
            }

            if (500U >= (stateTimer % ROP_RECORD_LAST_TANK_FILLING_FREQUENCY))
            {
               // Record the time of the tank filling (we do it regulary because the sequence can be interupted by standby, alarm or pause)
               DO_READ(ProductionProcessParameters.pProcessRecord->LastTankFillingTime, g_DoDate);
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }

            if(TRUE == isStopTankFillingConditions())
            {
               if(((UINT32)MOTOR_VALVE_INIT_FREQUENCY * (UINT32)ROP_HOUR_TO_MS) < MVM_GetLastInitTime())
               {
                  MVM_StartMotorValveInitialization();
               }

               // Sequence is finised, switch to WAITING mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Tank Filling", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceRoDiagnostic(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT drainValvePosition;
   FLOAT permeateFlowrateMeasure;
   FLOAT drainFlowrateMeasure;
   FLOAT roPumpFlowrate;
   
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      if(((UINT32)MOTOR_VALVE_INIT_FREQUENCY * (UINT32)ROP_HOUR_TO_MS) < MVM_GetLastInitTime())
      {
         MVM_StartMotorValveInitialization();
         rop_setNewState(p_ProcessParam, ROP_STATE_UF_WAIT); // Only to force the change of transition
      }

      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.StateNumber = 1U;
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;

      // Compute Remaining Time
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (((UINT32)TEST_PRESSURE_DURATION * 2U) + (UINT32)ROP_TEST_RO_PUMP_DURATION + (3U * 15U)); // 15 seconds to close drain (tap pressure) and recirc (test ro pump) valves
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_WAIT:
         {
            if(1U == p_ProcessParam->ProcessDescription.StateNumber)
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_TEST_TAP_PRESSURE);
               tapPressureMeasureEnable = FALSE;
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
            }
            else if(2U == p_ProcessParam->ProcessDescription.StateNumber)
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_TEST_RO_PUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)ROP_TEST_RO_PUMP_DURATION + 15U);
            }
            else
            {
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               rop_setAutoDiagnosticFlag (FALSE);
               // Sequence is finised, switch to WAITING or STANDBY mode
               if (ROP_MODE_DAILY_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode)
               {
                  rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               }
               else
               {
                  rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
               }
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            break;
         }

         case ROP_STATE_TEST_TAP_PRESSURE:
         {
            if(((UINT32)ROP_TEST_TAP_PRESSURE_DELAY <= stateTimer) && // Timer ok for mesure start
               (FALSE == tapPressureMeasureEnable))
            {
               // Start Tap Pressure Mesure
               MES_RecordStart(MES_TEST_TAP_FEED, TRUE);
               tapPressureMeasureEnable = TRUE;
            }
            if(((UINT32)TEST_PRESSURE_DURATION * 1000U) <= stateTimer) // Timer ok for mesure stop
            {
               // Stop Tap Pressure Mesure
               MES_RecordStop(MES_TEST_TAP_FEED);
               tapPressureMeasureEnable = FALSE;
               rop_sendTapFeedPressure();
               rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (((UINT32)TEST_PRESSURE_DURATION * 2U) + (UINT32)ROP_TEST_RO_PUMP_DURATION + (2U * 15U) - (UINT32)(stateTimer / 1000U));
            }
            break;
         }

         case ROP_STATE_SHORT_FLUSH:
         {
            if(((UINT32)ROP_TEST_TAP_PRESSURE_DELAY <= stateTimer) && // Timer ok for mesure start
               (FALSE == tapPressureMeasureEnable))
            {
               // Start Progard Delta P Mesure
               MES_RecordStart(MES_TEST_PROGARD, TRUE);
               MES_RecordStart(MES_PROGARD_DELTA_P_FLOWRATE, TRUE);
               tapPressureMeasureEnable = TRUE;
            }
            if(((UINT32)TEST_PRESSURE_DURATION * 1000U) <= stateTimer) // Timer ok for mesure stop
            {
               // Stop Progard Delta P Mesure
               MES_RecordStop(MES_TEST_PROGARD);
               MES_RecordStop(MES_PROGARD_DELTA_P_FLOWRATE);
               tapPressureMeasureEnable = FALSE;
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.StateNumber = 2U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)TEST_PRESSURE_DURATION + (UINT32)ROP_TEST_RO_PUMP_DURATION + 15U - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         case ROP_STATE_TEST_RO_PUMP:
         {
            if((UINT32)ROP_VOLUME_COUNTER_DELAY <= stateTimer)
            {
               drainValvePosition = MVM_GetPosition(MVM_VALVE_DRAIN);
               permeateFlowrateMeasure = MES_GetMeasure(MES_PERMEATE_FLOWRATE);
               drainFlowrateMeasure = MES_GetMeasure(MES_DRAIN_FLOWRATE);

               if((permeateFlowrateMeasure > (FLOAT)(SYS_GetPermeateFlowrate())) || // if Permeate Flow Rate > SP
                  ((FLOAT)RO_PRESSURE_MAX_TEST_RO_PUMP < MES_GetMeasure(MES_RO_PRESSURE)) || // or if RO Pressure is too high
                  (MES_GetFeedPressureMin() >= MES_GetMeasure(MES_RO_PUMP_INLET_PRESSURE)) || // or if RO Pump Inlet Pressure is too low
                  (3.0F >= drainValvePosition) || // or if the drain valve is already closed
                  (((UINT32)ROP_TEST_RO_PUMP_DURATION * 1000U) <= stateTimer)) // or if the state has last for more than ROP_TEST_RO_PUMP_DURATION
               {
                  roPumpFlowrate = (0xFFFFU);
                  // Record Pump Flow Rate and send Results PDO
                  if (permeateFlowrateMeasure > (FLOAT)(SYS_GetPermeateFlowrate()))
                  {
                     roPumpFlowrate = ((((permeateFlowrateMeasure+drainFlowrateMeasure)*1000.0F)/60.0F) + 0.5f); // Flowrate in mL/min
                     if (0.0F > roPumpFlowrate)
                     {
                        roPumpFlowrate = 0.0F;
                     }
                     else if (65534.0F < roPumpFlowrate)
                     {
                        roPumpFlowrate = 65534.0F;
                     }
                     else
                     {
                        /* Do Nothing */
                     }
                  }

                  rop_sendRODiagnosticResults ((UINT16)roPumpFlowrate);
                  rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
                  p_ProcessParam->ProcessDescription.StateNumber = 3U;
               }
               else
               {
                  // Close Drain Valve
                  MVM_SetValvePosition(MVM_VALVE_DRAIN, drainValvePosition - 1.0F); // We close the valve of 1% every 500ms
               }
            }
            if(((UINT32)ROP_TEST_RO_PUMP_DURATION * 1000U) >= stateTimer)
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)ROP_TEST_RO_PUMP_DURATION - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence RO Diagnostic", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceAlarmStop(ROP_ProcessParam_t * p_ProcessParam)
{
   FLOAT FeedFlowRate;
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_FAST_WAIT);
      rop_stopSpecialCounters();
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
   }

   // In Alarm Stop mode, the switch to other modes is managed in "rop_manageAlarmStopAndPause" function

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)
      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_FAST_WAIT:
         {
            if(((UINT32)TEST_LOW_PRESSURE_FREQUENCY * 1000U)<= stateTimer) // After TEST_LOW_PRESSURE_FREQUENCY
            {
               if (FALSE == rop_isAlarmStopPresent(FALSE, TRUE)) // Only Low feed Water Pressure Alarms
               {
                  // Switch to SHORT FLUSH MESURE
                  rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH);
               }
            }
            break;
         }

         case ROP_STATE_SHORT_FLUSH:
         {
            if (TRUE == rop_isAlarmStopPresent(FALSE, TRUE)) // Another alarm than Low feed Water Pressure Alarms is present
            {
               // Switch to WAIT
               rop_setNewState(p_ProcessParam, ROP_STATE_FAST_WAIT);
            }
            else if(((UINT32)TEST_LOW_PRESSURE_DURATION * 1000U) <= stateTimer) // After TEST_LOW_PRESSURE_DURATION
            {
               FeedFlowRate = MES_GetMeasure(MES_DRAIN_FLOWRATE) + MES_GetMeasure (MES_PERMEATE_FLOWRATE);
               if(FeedFlowRate >= ALARM_FEED_FLOWRATE_FLUSH_MIN) // Alarm is resetable
               {
                  AAA_ResetAlarm(ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_LOW);
                  AAA_ResetAlarm(ALARM_TYPE_FEED_PRESSURE_MIN);
               }
               rop_setNewState(p_ProcessParam, ROP_STATE_FAST_WAIT);
            }
            else
            {
               /* Do Nothing */
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Alarm Stop", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequencePause(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_FAST_WAIT);
      rop_stopSpecialCounters();
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }
      
      // Nothing else is done in Pause mode and the return to other mode is managed in "rop_manageAlarmStopAndPause" function
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceProgardFlush(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 FeedVolumeRemaining;
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), ProductionProcessSetpoints.RoRecoverySP);

      if(TRUE == rop_isProgardWithChlore()) // Consumable with Cl2
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_FLUSH);

         // Compute Remaining Time
         p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)ROP_SLOW_TRANSITION_DURATION + (UINT32)PROGARD_CL2_RINSE_DURATION + (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_SHORT_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)ROP_SLOW_TRANSITION_DURATION + rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) / 1000U;
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      VolumeCounterEnable = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_SLOW_FLUSH:
         {
            rop_StartFeedVolumeCounter(); // Counter will be start only at the first call
            if (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) > rop_getFeedVolumeCounter())
            {
               FeedVolumeRemaining = rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) - rop_getFeedVolumeCounter();
            }
            else
            {
               FeedVolumeRemaining = 0U;
            }

            if(0U == FeedVolumeRemaining) // After PROGARD_CL2_FLUSH_VOLUME
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               VolumeCounterEnable = FALSE;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)PROGARD_CL2_RINSE_DURATION;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)PROGARD_CL2_RINSE_DURATION + (FeedVolumeRemaining / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND));
            }
            break;
         }

         case ROP_STATE_WAIT:
         {
            rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)PROGARD_CL2_RINSE_DURATION;
            break;
         }

         case ROP_STATE_RINSE:
         {
            if(((UINT32)PROGARD_CL2_RINSE_DURATION * 1000U) <= stateTimer) // After PROGARD_CL2_RINSE_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Sequence is finised, switch to WAITING mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)PROGARD_CL2_RINSE_DURATION - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         case ROP_STATE_SLOW_SHORT_FLUSH:
         {
            if(rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) <= stateTimer) // After PROGARD_FLUSH_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Sequence is finised, switch to WAITING mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)((rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) - stateTimer) / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Progard Flush", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
   else
   {
      if ((ROP_STATE_SLOW_FLUSH == p_ProcessParam->ProcessDescription.ProcessState) ||
          (ROP_STATE_SLOW_SHORT_FLUSH == p_ProcessParam->ProcessDescription.ProcessState))
      {
         stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)
         if(p_ProcessParam->ProcessDescription.SequenceTotalDuration * 1000U >= stateTimer)
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (p_ProcessParam->ProcessDescription.SequenceTotalDuration - (stateTimer/1000U));
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceFirstInstall(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   UINT32 temporaryStateTimer;
   UINT32 FeedVolumeRemaining;
   UINT32 statePassedTime;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), (UINT8)DEFAULT_RO_RECOVERY);
      DO_WRITE(g_DoProductionSetpoints.IsTheFirstStartUpProd, FALSE); 
      p_ProcessParam->pProcessRecord->isFirstStartUp = FALSE;
      SYS_RefreshBackupRecord(SYS_REC_PROD);

      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)ROP_SLOW_TRANSITION_DURATION + (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;

      if(TRUE == rop_isProgardWithChlore()) // Consumable with Cl2
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration += (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_SHORT_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration += rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) / 1000U;
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      VolumeCounterEnable = FALSE;

      if (ROP_STATE_RINSE == p_ProcessParam->pProcessRecord->ProcessState)
      {
         // The sequence was interrupted in Rinse state, system shall restart from where it has been interrupted
         rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION - (UINT32)(p_ProcessParam->pProcessRecord->RecordParameter / 1000U);
         rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
      }
      else if(ROP_STATE_FLUSH == p_ProcessParam->pProcessRecord->ProcessState)
      {
         // The sequence was interrupted in Flush state, system shall restart from where it has been interrupted
         rop_setNewState(p_ProcessParam, ROP_STATE_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION) - (UINT32)(p_ProcessParam->pProcessRecord->RecordParameter / 1000U);
         rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
      }
      else if(ROP_STATE_TEST_TAP_PRESSURE == p_ProcessParam->pProcessRecord->ProcessState)
      {
         // switch to WAIT1 state
         rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
         p_ProcessParam->ProcessDescription.StateNumber = 1U;
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;
      }
      else
      {
        /* Start from the beginning */
      }

      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_SLOW_FLUSH:
         {
            rop_StartFeedVolumeCounter(); // Counter will be start only at the first call
            if (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) > rop_getFeedVolumeCounter())
            {
               FeedVolumeRemaining = rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) - rop_getFeedVolumeCounter();
            }
            else
            {
               FeedVolumeRemaining = 0U;
            }

            if(0U == FeedVolumeRemaining) // After PROGARD_CL2_FLUSH_VOLUME
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.StateNumber = 1U;
               VolumeCounterEnable = FALSE;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;

               // Now, process will restart in Wait 1 in case of interruption
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_TEST_TAP_PRESSURE;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime += (FeedVolumeRemaining / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
            }
            break;
         }
         case ROP_STATE_FLUSH:
         {
            temporaryStateTimer = TMC_GetTimeCounter(rop_TemporaryStateTimeCounter); // (ms)
            statePassedTime = p_ProcessParam->pProcessRecord->RecordParameter + temporaryStateTimer;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION) - (UINT32)(statePassedTime / 1000U);

            if ((UINT32)ROP_TIMER_RECORD_FREQUENCY <= temporaryStateTimer)
            {
               // Regularly, the timer is recorded in case of interruption
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
               p_ProcessParam->pProcessRecord->RecordParameter = statePassedTime;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }

            if(((UINT32)RO_CARTRIDGE_FLUSH_DURATION * 1000U) <= statePassedTime) // After RO_CARTRIDGE_FLUSH_DURATION
            {
               // Switch to WAIT3 state
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.StateNumber = 3U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION;

               // Now, process will restart in Rinse in case of interruption
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_RINSE;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }
            break;
         }

         case ROP_STATE_SLOW_SHORT_FLUSH:
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime += (UINT32)((rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) - stateTimer) / 1000U);

            if(rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) <= stateTimer) // After PROGARD_FLUSH_DURATION
            {
               // switch to WAIT1 state
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.StateNumber = 1U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;

               // Now, process will restart in Wait 1 in case of interruption
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_TEST_TAP_PRESSURE;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }
            break;
         }

         case ROP_STATE_WAIT:
         {
            if (1U == p_ProcessParam->ProcessDescription.StateNumber)
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_TEST_TAP_PRESSURE);
               tapPressureMeasureEnable = FALSE;
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
            }
            else if (2U == p_ProcessParam->ProcessDescription.StateNumber)
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_FLUSH);
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
            }
            else
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
            }
            break;
         }

         case ROP_STATE_TEST_TAP_PRESSURE:
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION + (UINT32)TEST_PRESSURE_DURATION;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime -= (UINT32)(stateTimer / 1000U);

            if(((UINT32)ROP_TEST_TAP_PRESSURE_DELAY  <= stateTimer) && // Timer ok for mesure start (2 seconds delay)
               (FALSE == tapPressureMeasureEnable))
            {
               // Start Tap Pressure Mesure
               MES_RecordStart(MES_TEST_TAP_FEED, TRUE);
               tapPressureMeasureEnable = TRUE;
            }

            if(((UINT32)TEST_PRESSURE_DURATION * 1000U) <= stateTimer) // Timer ok for mesure stop
            {
               // Stop Tap Pressure Measure
               MES_RecordStop(MES_TEST_TAP_FEED);
               tapPressureMeasureEnable = FALSE;
               rop_sendTapFeedPressure();

               // switch to WAIT2 state
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.StateNumber = 2U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION;

               // Now, process will restart in Flush in case of interruption
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_FLUSH;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
            }
            break;
         }

         case ROP_STATE_RINSE:
         {
            temporaryStateTimer = TMC_GetTimeCounter(rop_TemporaryStateTimeCounter); // (ms)
            statePassedTime = p_ProcessParam->pProcessRecord->RecordParameter + temporaryStateTimer;

            if ((UINT32)ROP_TIMER_RECORD_FREQUENCY <= temporaryStateTimer)
            {
               // Regularly, the timer is recorded in case of interruption
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
               p_ProcessParam->pProcessRecord->RecordParameter = statePassedTime;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }

            if(((UINT32)RO_CARTRIDGE_RINSE_DURATION * 1000U) <= statePassedTime) // After RO_CARTRIDGE_RINSE_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Sequence is finised, switch to WAITING mode
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_WAIT;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION - (UINT32)(statePassedTime / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence First Install", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
   else
   {
      if ((ROP_STATE_SLOW_FLUSH == p_ProcessParam->ProcessDescription.ProcessState) ||
          (ROP_STATE_SLOW_SHORT_FLUSH == p_ProcessParam->ProcessDescription.ProcessState))
      {
         stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)
         if(p_ProcessParam->ProcessDescription.SequenceTotalDuration * 1000U >= stateTimer)
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (p_ProcessParam->ProcessDescription.SequenceTotalDuration - (stateTimer/1000U));
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceRoRinse(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 temporaryStateTimer;
   UINT32 statePassedTime;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), (UINT8)DEFAULT_RO_RECOVERY);

      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION;

      if (ROP_STATE_RINSE == p_ProcessParam->pProcessRecord->ProcessState)
      {
         // The process shall restart from the Rinse where it has been interrupted
         rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION - (UINT32)(p_ProcessParam->pProcessRecord->RecordParameter / 1000U);
         rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION) - (UINT32)(p_ProcessParam->pProcessRecord->RecordParameter / 1000U);
         rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
      }
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_FLUSH:
         {
            temporaryStateTimer = TMC_GetTimeCounter(rop_TemporaryStateTimeCounter); // (ms)
            statePassedTime = p_ProcessParam->pProcessRecord->RecordParameter + temporaryStateTimer;

            if ((UINT32)ROP_TIMER_RECORD_FREQUENCY <= temporaryStateTimer)
            {
               // Regularly, the timer is recorded in case of interruption
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
               p_ProcessParam->pProcessRecord->RecordParameter = statePassedTime;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }

            if(((UINT32)RO_CARTRIDGE_FLUSH_DURATION * 1000U) <= statePassedTime) // After RO_CARTRIDGE_FLUSH_DURATION
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION;

               // In case of interruption, the system will restart in Rinse state
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_RINSE;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)RO_CARTRIDGE_RINSE_DURATION + (UINT32)RO_CARTRIDGE_FLUSH_DURATION) - (UINT32)(statePassedTime / 1000U);
            }
            break;
         }

         case ROP_STATE_WAIT:
         {
            rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
            rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
            break;
         }

         case ROP_STATE_RINSE:
         {
            temporaryStateTimer = TMC_GetTimeCounter(rop_TemporaryStateTimeCounter); // (ms)
            statePassedTime = p_ProcessParam->pProcessRecord->RecordParameter + temporaryStateTimer;

            if ((UINT32)ROP_TIMER_RECORD_FREQUENCY <= temporaryStateTimer)
            {
               // Regularly, the timer is recorded in case of interruption
               rop_TemporaryStateTimeCounter = TMC_StartTimeCounter();
               p_ProcessParam->pProcessRecord->RecordParameter = statePassedTime;
               SYS_RefreshBackupRecord(SYS_REC_PROD);
            }

            if(((UINT32)RO_CARTRIDGE_RINSE_DURATION * 1000U) <= statePassedTime) // After RO_CARTRIDGE_RINSE_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Sequence is finised, switch to WAITING mode
               p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_WAIT;
               p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_CARTRIDGE_RINSE_DURATION - (UINT32)(statePassedTime / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence RO Rinse", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceCl2Cleaning(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   UINT32 FeedVolumeRemaining;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_FLUSH);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      VolumeCounterEnable = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (rop_getSystemDependentParameters(RO_CL2_FLUSH_VOLUME) / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;

   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_FLUSH:
         {
            if((UINT32)ROP_VOLUME_COUNTER_DELAY <= stateTimer) // Start Volume Counter after 2 seconds
            {
               rop_StartFeedVolumeCounter(); // Counter will be start only at the first call

               if (rop_getSystemDependentParameters(RO_CL2_FLUSH_VOLUME) > rop_getFeedVolumeCounter())
               {
                  FeedVolumeRemaining = rop_getSystemDependentParameters(RO_CL2_FLUSH_VOLUME) - rop_getFeedVolumeCounter();
               }
               else
               {
                  FeedVolumeRemaining = 0U;
               }
                  
               if(0U == FeedVolumeRemaining) // After RO_CL2_FLUSH_VOLUME
               {
                  // Sequence is finised, switch to REAGENT REMOVAL mode
                  rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
                  VolumeCounterEnable = FALSE;
                  p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
                  p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
               }
               else
               {
                  p_ProcessParam->ProcessDescription.SequenceRemainingTime = (FeedVolumeRemaining / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
               }
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Cl2 Cleaning", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequencePhCleaning(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   UINT32 FeedVolumeRemaining;
   UINT8 nbrCartridges;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      VolumeCounterEnable = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)RO_PH_FLUSH_DURATION + (UINT32)RO_PH_SOAKING_DURATION + (rop_getSystemDependentParameters(RO_PH_FLUSH_VOLUME) / (UINT32)ROP_LIMITED_FLUSH_FLOWRATE_BY_SECOND);

      // Set Drain Valve Position
      nbrCartridges = SYS_GetNbCartridges();
      if (SYSTEM_ONE_CARTRIDGE == nbrCartridges)
      {
         p_ProcessParam->TransitionDescription.LimitedFlushDrainValvePosition = (FLOAT)RO_PH_DRAIN_VALVE_OPENING_1_CARTRIDGE; // %
      }
      else if (SYSTEM_TWO_CARTRIDGES == nbrCartridges)
      {
         p_ProcessParam->TransitionDescription.LimitedFlushDrainValvePosition = (FLOAT)RO_PH_DRAIN_VALVE_OPENING_2_CARTRIDGES; // %
      }
      else if (SYSTEM_THREE_CARTRIDGES == nbrCartridges)
      {
         p_ProcessParam->TransitionDescription.LimitedFlushDrainValvePosition = (FLOAT)RO_PH_DRAIN_VALVE_OPENING_3_CARTRIDGES; // %
      }
      else if (SYSTEM_FOUR_CARTRIDGES == nbrCartridges)
      {
         p_ProcessParam->TransitionDescription.LimitedFlushDrainValvePosition = (FLOAT)RO_PH_DRAIN_VALVE_OPENING_4_CARTRIDGES; // %
      }
      else
      {
         ERROR("Unknown Nbr of Cartridges configuration: %d", nbrCartridges);
      }
 
      if(TRUE == cleaningSequenceStartedFromTheStart) // Operator Order: Start from the beginning
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_LIMITED_FLUSH_DEAD_END);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
         cleaningSequenceStartedFromTheStart = FALSE;
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_PH_FLUSH_DURATION;
      }
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_LIMITED_FLUSH_DEAD_END:
         {
            rop_StartFeedVolumeCounter(); // Counter will be start only at the first call
            if (rop_getSystemDependentParameters(RO_PH_FLUSH_VOLUME) > rop_getFeedVolumeCounter())
            {
               FeedVolumeRemaining = rop_getSystemDependentParameters(RO_PH_FLUSH_VOLUME) - rop_getFeedVolumeCounter();
            }
            else
            {
               FeedVolumeRemaining = 0U;
            }

            if(0U == FeedVolumeRemaining) // After RO_PH_FLUSH_VOLUME
            {
               // Switch to SOAKING
               p_ProcessParam->TransitionDescription.LimitedFlushDrainValvePosition = 0.0F;
               rop_setNewState(p_ProcessParam, ROP_STATE_SOAKING);
               VolumeCounterEnable = FALSE;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_PH_FLUSH_DURATION + (UINT32)RO_PH_SOAKING_DURATION + (FeedVolumeRemaining / (UINT32)ROP_LIMITED_FLUSH_FLOWRATE_BY_SECOND);
            }
            break;
         }

         case ROP_STATE_SOAKING:
         {
            if(((UINT32)RO_PH_SOAKING_DURATION * 1000U) <= stateTimer) // After RO_PH_SOAKING_DURATION
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((UINT32)RO_PH_FLUSH_DURATION + (UINT32)RO_PH_SOAKING_DURATION) - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         case ROP_STATE_SHORT_FLUSH:
         {
            if(((UINT32)RO_PH_FLUSH_DURATION * 1000U) <= stateTimer) // After RO_PH_FLUSH_DURATION
            {
               // Sequence is finised, switch to REAGENT REMOVAL mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)RO_PH_FLUSH_DURATION - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence pH Cleaning", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceReagentRemoval(ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   UINT32 FeedVolumeRemaining;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), ProductionProcessSetpoints.RoRecoverySP);

      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)ROP_SLOW_TRANSITION_DURATION + (UINT32)REAGENT_REMOVAL_RINSE_DURATION;

      if(TRUE == rop_isProgardWithChlore()) // Consumable with Cl2
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration += (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      else
      {
         rop_setNewState(p_ProcessParam, ROP_STATE_SLOW_SHORT_FLUSH);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration += rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) / 1000U;
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }

      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      VolumeCounterEnable = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_SLOW_FLUSH:
         {
            rop_StartFeedVolumeCounter(); // Counter will be start only at the first call
            if (rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) > rop_getFeedVolumeCounter())
            {
               FeedVolumeRemaining = rop_getSystemDependentParameters(PROGARD_CL2_FLUSH_VOLUME) - rop_getFeedVolumeCounter();
            }
            else
            {
               FeedVolumeRemaining = 0U;
            }

            if(0U == FeedVolumeRemaining) // After PROGARD_CL2_FLUSH_VOLUME
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               VolumeCounterEnable = FALSE;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)REAGENT_REMOVAL_RINSE_DURATION;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)REAGENT_REMOVAL_RINSE_DURATION + (FeedVolumeRemaining / (UINT32)ROP_FLUSH_FLOWRATE_BY_SECOND);
            }
            break;
         }

         case ROP_STATE_SLOW_SHORT_FLUSH:
         {
            if(rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) <= stateTimer) // After PROGARD_FLUSH_DURATION
            {
               rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)REAGENT_REMOVAL_RINSE_DURATION;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)REAGENT_REMOVAL_RINSE_DURATION + (UINT32)((rop_getSystemDependentParameters(PROGARD_FLUSH_DURATION) - stateTimer) / 1000U);
            }
            break;
         }

         case ROP_STATE_WAIT:
         {
            rop_setNewState(p_ProcessParam, ROP_STATE_RINSE);
            break;
         }

         case ROP_STATE_RINSE:
         {
            if(((UINT32)REAGENT_REMOVAL_RINSE_DURATION * 1000U) <= stateTimer) // After REAGENT_REMOVAL_RINSE_DURATION
            {
               // Reset Flush Timer
               rop_resetFlushTimer(p_ProcessParam);
               // Sequence is finised, switch to WAITING mode
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)REAGENT_REMOVAL_RINSE_DURATION - (UINT32)(stateTimer / 1000U);
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Reagent Removal", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
   else
   {
      if ((ROP_STATE_SLOW_FLUSH == p_ProcessParam->ProcessDescription.ProcessState) ||
          (ROP_STATE_SLOW_SHORT_FLUSH == p_ProcessParam->ProcessDescription.ProcessState))
      {
         stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)
         if(p_ProcessParam->ProcessDescription.SequenceTotalDuration * 1000U >= stateTimer)
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (p_ProcessParam->ProcessDescription.SequenceTotalDuration - (stateTimer/1000U));
         }
      }
   }
}


/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceTest(ROP_ProcessParam_t * p_ProcessParam)
{
   FLOAT fRecircPosition;
   FLOAT fDrainPosition;
   
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT_TEST);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      DO_WRITE(g_DoRecirculationMotorValve.TestForcedPosition, 100.0F);
      DO_WRITE(g_DoDrainMotorValve.TestForcedPosition, 100.0F);
   }
   
   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      DO_READ(fRecircPosition, g_DoRecirculationMotorValve.TestForcedPosition);
      DO_READ(fDrainPosition, g_DoDrainMotorValve.TestForcedPosition);
      
      if ((0.0F <= fRecircPosition) &&
          (100.0F >= fRecircPosition))
      {
         MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, fRecircPosition);
      }
      
      if ((0.0F <= fDrainPosition) &&
          (100.0F >= fDrainPosition))
      {
         MVM_SetValvePosition(MVM_VALVE_DRAIN, fDrainPosition);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceTestRinse(ROP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      // Set the regulation setpoints
      MVM_SetRegulationSetpoint(SYS_GetPermeateFlowrate(), (UINT8)DEFAULT_RO_RECOVERY);

      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_WAIT:
         {
            rop_setNewState(p_ProcessParam, ROP_STATE_RINSE_MESURE);
            break;
         }

         case ROP_STATE_RINSE_MESURE:
         {
            /* Do Nothing */
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Test Rinse", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID rop_manageSequenceTestFlush(ROP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      rop_setNewState(p_ProcessParam, ROP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_WAIT:
         {
            rop_setNewState(p_ProcessParam, ROP_STATE_SHORT_FLUSH_MEASURE);
            break;
         }

         case ROP_STATE_SHORT_FLUSH_MEASURE:
         {
            /* Do Nothing */
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Test Flush", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param SystemDependentParameters [IN] SystemDependentParameters_e * <br>
 *                    Type of the parameters
 *
 *  @return UINT32
 *  @retval Value of the parameters (Volume in mL and duration in msec)
 *
*  @brief  Return the value of a parameters dependent
 *
 */
static UINT32 rop_getSystemDependentParameters(SystemDependentParameters_e SystemDependentParameters)
{
   UINT32 RetunValue = 0;
   UINT8 nbrCartridges;

   switch(SystemDependentParameters)
   {
      case RO_POLYPHOSPHATE_REMOVAL_DURATION:
      {
         if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
         {
            RetunValue = (UINT32)RO_POLYPHOSPHATE_REMOVAL_DURATION_1_PROGARD * 1000U;
         }
         else
         {
            RetunValue = (UINT32)RO_POLYPHOSPHATE_REMOVAL_DURATION_2_PROGARDS * 1000U;
         }
         break;
      }

      case PROGARD_FLUSH_DURATION:
      {
         if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
         {
            RetunValue = (UINT32)PROGARD_FLUSH_DURATION_1_PROGARD * 1000U;
         }
         else
         {
            RetunValue = (UINT32)PROGARD_FLUSH_DURATION_2_PROGARDS * 1000U;
         }
         break;
      }

      case PROGARD_CL2_FLUSH_VOLUME:
      {
         if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
         {
            RetunValue = (UINT32)PROGARD_CL2_FLUSH_VOLUME_1_PROGARD * 1000U; // mL
         }
         else
         {
            RetunValue = (UINT32)PROGARD_CL2_FLUSH_VOLUME_2_PROGARDS * 1000U; // mL
         }
         break;
      }

      case RO_CL2_FLUSH_VOLUME:
      {
         nbrCartridges = SYS_GetNbCartridges();
         if (SYSTEM_ONE_CARTRIDGE == nbrCartridges)
         {
            RetunValue = (UINT32)RO_CL2_FLUSH_VOLUME_1_CARTRIDGE * 1000U; // mL
         }
         else if (SYSTEM_TWO_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_CL2_FLUSH_VOLUME_2_CARTRIDGES * 1000U; // mL
         }
         else if (SYSTEM_THREE_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_CL2_FLUSH_VOLUME_3_CARTRIDGES * 1000U; // mL
         }
         else if (SYSTEM_FOUR_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_CL2_FLUSH_VOLUME_4_CARTRIDGES * 1000U; // mL
         }
         else
         {
            ERROR("Unknown Nbr of Cartridges configuration: %d", nbrCartridges);
         }
         break;
      }

      case RO_PH_FLUSH_VOLUME:
      {
         nbrCartridges = SYS_GetNbCartridges();
         if (SYSTEM_ONE_CARTRIDGE == nbrCartridges)
         {
            RetunValue = (UINT32)RO_PH_FLUSH_VOLUME_1_CARTRIDGE; // mL
         }
         else if (SYSTEM_TWO_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_PH_FLUSH_VOLUME_2_CARTRIDGES; // mL
         }
         else if (SYSTEM_THREE_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_PH_FLUSH_VOLUME_3_CARTRIDGES; // mL
         }
         else if (SYSTEM_FOUR_CARTRIDGES == nbrCartridges)
         {
            RetunValue = (UINT32)RO_PH_FLUSH_VOLUME_4_CARTRIDGES; // mL
         }
         else
         {
            ERROR("Unknown Nbr of Cartridges configuration: %d", nbrCartridges);
         }
         break;
      }

      default:
      {
         ERROR("Unknown SystemDependentParameters");
         break;
      }
   }

   return(RetunValue);
}

/*!
 *
 *  @return none
 *
 *  @brief  Start the Feed Volume Counters
 *
 */
static VOID rop_StartFeedVolumeCounter (VOID)
{
   if (FALSE == VolumeCounterEnable)
   {
      PermeateCounterRef = MES_VolumeCounterGet(MES_PERMEATE_FLOWRATE);
      DrainCounterRef = MES_VolumeCounterGet(MES_DRAIN_FLOWRATE);
      VolumeCounterEnable = TRUE;
   }
}

/*!
 *
 *  @return none
 *
 *  @brief  Get the Feed Volume Passed
 *
 */
static UINT32 rop_getFeedVolumeCounter (VOID)
{
   UINT32 FeedVolumePassed = 0U;
   DOUBLE fFeedVolumePassed;
   /*! @warning Volume counter api change */
   DOUBLE DrainVolume = MES_VolumeCounterGet(MES_DRAIN_FLOWRATE); // (mL)
   DOUBLE PermeateVolume = MES_VolumeCounterGet(MES_PERMEATE_FLOWRATE); // (mL)
   if (TRUE == VolumeCounterEnable)
   {
      if((DrainVolume >= DrainCounterRef) &&
         (PermeateVolume >= PermeateCounterRef))
      {
         fFeedVolumePassed = (((DrainVolume + PermeateVolume) - DrainCounterRef) - PermeateCounterRef);
         FeedVolumePassed = (UINT32)fFeedVolumePassed;
      }
   }

   return(FeedVolumePassed); // (mL)
}

/*!
 *
 *  @return none
 *
 *  @brief  Stop the special counters (mostly RO-Diagnostic results and tank level saturation)
 *
 */
static VOID rop_stopSpecialCounters (VOID)
{
   MES_RecordStop(MES_TEST_TAP_FEED);
   tapPressureMeasureEnable = FALSE;
   MES_RecordStop(MES_TEST_PROGARD);
   MES_RecordStop(MES_PROGARD_DELTA_P_FLOWRATE);
   MES_TankLevelSaturation(FALSE);
}

/*!
 *  @param CheckType [IN] TypeOfCheck_e <br>
 *                    Type of check (depend on the process state)
 *
 *  @param isEnable [IN] BOOL <br>
 *                    Enable or Disable
 *
 *  @return none
 *
 *  @brief  Start or Stop the Alarm and Measure linked to a process state
 *
 */
VOID rop_startMeasureRecordAndAlarm (TypeOfCheck_e CheckType, BOOL isEnable)
{
   if (isEnable != CheckEnable[CheckType])
   {
      switch(CheckType)
      {
         case CHECK_ALARM_WAIT:
         {
            if(TRUE == isEnable)
            {
               // Check Feed Flowrate
               MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_FR_STANDBY);
               MES_AlarmChkStart(MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE);
            }
            else
            {
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_FR_STANDBY);
               MES_AlarmChkStop(MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE);
            }
            break;
         }

         case CHECK_ALARM_FLUSH:
         {
            if(TRUE == isEnable)
            {
               // Check Feed Flowrate
               MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_FR_FLUSH);
            }
            else
            {
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_FR_FLUSH);
            }
            break;
         }

         case CHECK_MEASURE_FEED_MIN_MAX:
         {
            if(TRUE == isEnable)
            {
               // Check Feed Conductivity
               MES_AlarmChkStart(MES_CHK_ALM_FEED_COND_MIN);
               MES_AlarmChkStart(MES_CHK_ALM_FEED_COND_MAX);
            }
            else
            {
               MES_AlarmChkStop(MES_CHK_ALM_FEED_COND_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_FEED_COND_MAX);
            }
            break;
         }
         
         case CHECK_MEASURE_FLUSH_SP:
         {
            if(TRUE == isEnable)
            {
               // Record and Check Tap Feed Conductivity
               MES_AlarmChkStart(MES_CHK_ALM_FEED_COND_SP);
               MES_RecordStart(MES_TAP_FEED_CONDUCTIVITY, FALSE);
            }
            else
            {
               MES_AlarmChkStop(MES_CHK_ALM_FEED_COND_SP);
               MES_RecordStop(MES_TAP_FEED_CONDUCTIVITY);
            }
            break;
         }

         case CHECK_ALARM_PUMP:
         {
            if(TRUE == isEnable)
            {
               // Check RO Pump Feed Pressure Min
               MES_AlarmChkStart(MES_CHK_ALM_FEED_PRESSURE_MIN);
            }
            else
            {
               // Check RO Pump Feed Pressure Min
               MES_AlarmChkStop(MES_CHK_ALM_FEED_PRESSURE_MIN);
            }
            break;
         }

         case CHECK_ALARM_RINSE:
         {
            /* CCR 357
            if(TRUE == isEnable)
            {
               if(FALSE == ProductionProcessSetpoints.isManualRegulationActivated)
               {
                  // Check Regulation
                  MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_FR_SP_MIN);
                  MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_FR_SP_MAX);
                  MES_AlarmChkStart(MES_CHK_ALM_RO_RECOVERY_SP_MIN);
               }
            }
            else
            {
               // Check Regulation
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_FR_SP_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_FR_SP_MAX);
               MES_AlarmChkStop(MES_CHK_ALM_RO_RECOVERY_SP_MIN);
            }
            */

            if(TRUE == isEnable)
            {
               // Record and Check RO Temperature
               MES_RecordStart(MES_RO_TEMPERATURE, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_FEED_TEMP_SP);
            }
            else
            {
               // Record and Check RO Temperature
               MES_RecordStop(MES_RO_TEMPERATURE);
               MES_AlarmChkStop(MES_CHK_ALM_FEED_TEMP_SP);
            }
            break;
         }

         case CHECK_MEASURE_RINSE:
         {
            if(TRUE == isEnable)
            {
               // Record and Check Feed Conductivity
               MES_RecordStart(MES_RO_FEED_CONDUCTIVITY, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_FEED_COND_MIN);
               MES_AlarmChkStart(MES_CHK_ALM_FEED_COND_MAX);

               // Record and Check Permeate Conductivity
               MES_RecordStart(MES_PERMEATE_CONDUCTIVITY, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_COND_MIN);
               MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_COND_MAX);

               // Record and Check FCE
               MES_RecordStart(MES_FCE, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_FCE_SP);

               // Record and Check RO Rejection
               MES_RecordStart(MES_RO_REJECTION, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_RO_REJECTION_SP);

               // Record Regulation parameters
               MES_RecordStart(MES_PERMEATE_FLOWRATE, FALSE);
               MES_RecordStart(MES_RO_RECOVERY, FALSE);
            }
            else
            {
               // Record and Check Feed Conductivity
               MES_RecordStop(MES_RO_FEED_CONDUCTIVITY);
               MES_AlarmChkStop(MES_CHK_ALM_FEED_COND_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_FEED_COND_MAX);

               // Record and Check Permeate Conductivity
               MES_RecordStop(MES_PERMEATE_CONDUCTIVITY);
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_COND_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_COND_MAX);

               // Record and Check FCE
               MES_RecordStop(MES_FCE);
               MES_AlarmChkStop(MES_CHK_ALM_FCE_SP);

               // Record and Check RO Rejection
               MES_RecordStop(MES_RO_REJECTION);
               MES_AlarmChkStop(MES_CHK_ALM_RO_REJECTION_SP);

               // Record Regulation parameters
               MES_RecordStop(MES_PERMEATE_FLOWRATE);
               MES_RecordStop(MES_RO_RECOVERY);
            }
            break;
         }

         case CHECK_ALARM_TANK_FILLING:
         {
            if(TRUE == isEnable)
            {
               if(FALSE == ProductionProcessSetpoints.isManualRegulationActivated)
               {
                  // Check RO Pressure Min
                  MES_AlarmChkStart(MES_CHK_ALM_RO_PRESSURE_MIN);

                  MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_EDI_FR_MIN);
                  MES_AlarmChkStart(MES_CHK_ALM_PERMEATE_EDI_FR_MAX);
                  MES_AlarmChkStart(MES_CHK_ALM_RO_RECOVERY_SP_MAX);
               }
            }
            else
            {
               // Check RO Pressure Min
               MES_AlarmChkStop(MES_CHK_ALM_RO_PRESSURE_MIN);

               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_EDI_FR_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_PERMEATE_EDI_FR_MAX);
               MES_AlarmChkStop(MES_CHK_ALM_RO_RECOVERY_SP_MAX);
            }
            break;
         }

         case CHECK_MEASURE_QUALITY_TANK_FILLING:
         {
            if(TRUE == isEnable)
            {
               // Record and Check EDI Resistivity and Temperature
               MES_RecordStart(MES_EDI_RESISTIVITY, FALSE);
               MES_RecordStart(MES_ELIX_TEMPERATURE, FALSE);
               MES_AlarmChkStart(MES_CHK_ALM_ELIX_RES_MIN);
               MES_AlarmChkStart(MES_CHK_ALM_ELIX_RES_MAX);
               MES_AlarmChkStart(MES_CHK_ALM_ELIX_RES_SP);
            }
            else
            {
               // Record and Check EDI Resistivity and Temperature
               MES_RecordStop(MES_EDI_RESISTIVITY);
               MES_RecordStop(MES_ELIX_TEMPERATURE);
               MES_AlarmChkStop(MES_CHK_ALM_ELIX_RES_MIN);
               MES_AlarmChkStop(MES_CHK_ALM_ELIX_RES_MAX);
               MES_AlarmChkStop(MES_CHK_ALM_ELIX_RES_SP);
            }
            break;
         }

         case CHECK_MEASURE_HYDRAULIC_TANK_FILLING:
         {
            if(TRUE == isEnable)
            {
               // Record RO Pressure and Inlet Pressure
               MES_RecordStart(MES_RO_PRESSURE, FALSE);
               MES_RecordStart(MES_RO_PUMP_INLET_PRESSURE, FALSE);

               // Record EDI Performance
               MES_RecordStart(MES_EDI_PRODUCTION_FLOWRATE, FALSE);
               MES_RecordStart(MES_EDI_RECOVERY, FALSE);

               // Record RO Membrane permeability
               MES_RecordStart(MES_RO_PERMEABILITY, FALSE);
            }
            else
            {
               // Record RO Pressure and Inlet Pressure
               MES_RecordStop(MES_RO_PRESSURE);
               MES_RecordStop(MES_RO_PUMP_INLET_PRESSURE);

               // Record EDI Performance
               MES_RecordStop(MES_EDI_PRODUCTION_FLOWRATE);
               MES_RecordStop(MES_EDI_RECOVERY);

               // Record RO Membrane permeability
               MES_RecordStop(MES_RO_PERMEABILITY);
            }
            break;
         }

         default:
         {
            ERROR("Unknown CheckType: %d", CheckType);
            break;
         }
      }

      CheckEnable[CheckType] = isEnable;
   }
}

/*!
 *  @param Timer [IN] UINT32 <br>
 *                   state Timer
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the call to hardware diagnostic when the process is OFF
 *          NOTE: Call the function only if the process is stopped 
 */
static VOID rop_manageTestActuators (UINT32 Timer)
{
   if (((UINT32)ROP_ACTUATORS_TEST_FREQUENCY <= Timer) &&
       (500U >= (Timer % (UINT32)ROP_ACTUATORS_TEST_FREQUENCY)))
   {
      ACT_GetDiagnosticReport(ACT_DIAG_PRODUCTION);
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the start of alarm and measure associated to a process state
 *
 */
static VOID rop_manageStartMeasureAndRecord (ROP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
                                  
   if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case ROP_STATE_WAIT:
         {
            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY*1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, TRUE);// Start Alarm Check
            }
            rop_manageTestActuators(stateTimer);
            break;
         }

         case ROP_STATE_RINSE:
         {
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, TRUE);// Start Check RO Pump Feed Pressure
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, TRUE);// Start Alarm Low RO Pressure
            
            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY * 1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, TRUE);// Start Alarm
            }
            break;
         }

         case ROP_STATE_RINSE_MESURE:
         {
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, TRUE);// Start Check RO Pump Feed Pressure
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, TRUE);// Start Alarm Low RO Pressure

            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY*1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, TRUE);// Start Alarm
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, TRUE);// Start Mesure
            }

            if(((UINT32)RINSE_DURATION_MIN * 1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, TRUE);// Start Mesure
            }
            break;
         }

         case ROP_STATE_TANK_FILLING:
         {
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, TRUE);// Start Check RO Pump Feed Pressure
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, TRUE);// Start Alarm Low RO Pressure

            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY * 1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, TRUE);// Start Alarm
            }
            break;
         }

         case ROP_STATE_TANK_FILLING_MESURE:
         {
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, TRUE);// Start Check RO Pump Feed Pressure
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, TRUE);// Start Alarm Low RO Pressure

            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY*1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, TRUE);// Start Alarm
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, TRUE);// Start Mesure
            }

            if(((UINT32)RINSE_DURATION_MIN * 1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, TRUE);// Start Mesure
            }

            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY * 1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, TRUE);// Start Mesure
            }

            if (((UINT32)START_REGULATION_MEASURE_DELAY * 1000U) <= stateTimer) // After delay for regulation
            {
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, TRUE);// Start Mesure
            }
            break;
         }

         case ROP_STATE_LIMITED_FLUSH_DEAD_END:
         {
            // Set Flush Alarm to Limited
            MES_SetFlushLimited(TRUE);
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, TRUE);// Start Check Flush Flowrate Min
            break;
         }

         case ROP_STATE_FLUSH:
         case ROP_STATE_SHORT_FLUSH:
         case ROP_STATE_SLOW_FLUSH:
         case ROP_STATE_SLOW_SHORT_FLUSH:
         {
            // Set Flush Alarm to Not Limited
            MES_SetFlushLimited(FALSE);
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, TRUE);// Start Check Flush Flowrate Min
            break;
         }

         case ROP_STATE_SHORT_FLUSH_MEASURE:
         {
            // Set Flush Alarm to Not Limited
            MES_SetFlushLimited(FALSE);
            rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, TRUE);// Start Check Flush Flowrate Min

            if(((UINT32)START_WATER_QUALITY_MEASURE_DELAY*1000U) <= stateTimer) // Timer ok for mesure
            {
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, TRUE);// Start Mesure
               rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, TRUE);// Start Mesure
            }
            break;
         }

         case ROP_STATE_FAST_WAIT:
         {
            rop_manageTestActuators(stateTimer);
            break;
         }
         case ROP_STATE_UF_WAIT:
         case ROP_STATE_SOAKING:
         case ROP_STATE_TEST_TAP_PRESSURE:
         case ROP_STATE_TEST_RO_PUMP:
         case ROP_STATE_SAN_RINSE:
         case ROP_STATE_SAN_TANK_FILLING:
         case ROP_STATE_WAIT_TEST:
         default:
         {
            break;
         }
      }
   }
}

/*!
 *  @param newState [IN] RopState_e <br>
 *                    Process State
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the stop of alarm and measure associated to a process state
 *
 */
static VOID rop_manageStopMeasureAndRecord (RopState_e newState)
{
   switch(newState)
   {
      case ROP_STATE_WAIT:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_RINSE:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_RINSE_MESURE:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_TANK_FILLING:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_TANK_FILLING_MESURE:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         break;
      }

      case ROP_STATE_FLUSH:
      case ROP_STATE_LIMITED_FLUSH_DEAD_END:
      case ROP_STATE_SHORT_FLUSH:
      case ROP_STATE_SLOW_FLUSH:
      case ROP_STATE_SLOW_SHORT_FLUSH:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_SHORT_FLUSH_MEASURE:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      case ROP_STATE_SOAKING:
      case ROP_STATE_TEST_TAP_PRESSURE:
      case ROP_STATE_TEST_RO_PUMP:
      case ROP_STATE_SAN_RINSE:
      case ROP_STATE_SAN_TANK_FILLING:
      case ROP_STATE_FAST_WAIT:
      case ROP_STATE_UF_WAIT:
      case ROP_STATE_WAIT_TEST:
      {
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_WAIT, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_FLUSH, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FEED_MIN_MAX, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_FLUSH_SP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_RINSE, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_ALARM_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_QUALITY_TANK_FILLING, FALSE);
         rop_startMeasureRecordAndAlarm(CHECK_MEASURE_HYDRAULIC_TANK_FILLING, FALSE);
         break;
      }

      default:
      {
         break;
      }
   }
}


/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the manual regulation
 *
 */
static VOID rop_manageManualRegulation(ROP_ProcessParam_t * p_ProcessParam)
{
   FLOAT fRecircPosition;
   FLOAT fDrainPosition;

   if(TRUE == ProductionProcessSetpoints.isManualRegulationActivated)
   {
      if((ROP_STATE_RINSE == p_ProcessParam->ProcessDescription.ProcessState)          ||
         (ROP_STATE_RINSE_MESURE == p_ProcessParam->ProcessDescription.ProcessState)   ||
         (ROP_STATE_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessState)   ||
         (ROP_STATE_TANK_FILLING_MESURE == p_ProcessParam->ProcessDescription.ProcessState))
      {
         if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
         {
            DO_READ(fRecircPosition, g_DoRecirculationMotorValve.RegulationForcedPosition);
            DO_READ(fDrainPosition, g_DoDrainMotorValve.RegulationForcedPosition);

            if ((0.0F <= fRecircPosition) &&
                (100.0F >= fRecircPosition))
            {
               MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, fRecircPosition);
            }

            if ((0.0F <= fDrainPosition) &&
                (100.0F >= fDrainPosition))
            {
               MVM_SetValvePosition(MVM_VALVE_DRAIN, fDrainPosition);
            }
         }
      }
   }
}

/*!
 * @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

