/*!
* @file rop_task.c
*
* @brief Production Task
*
* @details Contain all Production stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: rop_task.c 539 2014-08-21 08:13:2#$: Filename
* $Rev:: 539                              $: Revision of last commit
* $Author:: m175711                       $: Author of last commit
* $Date:: 2014-08-21 10:13:24 +0200 (jeu.#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/production/src/rop_task.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/
#define  DELAY500MS     ((UINT32)500)


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "messaging.h"
#include "rop_api.h"
#include "rop_rec.h"
#include "sys_softerr.h"
#include "rop_data.h"
#include "mes_api.h"
#include "sys_api.h"
#include "sys_log.h"
#include "rop_private.h"
#include "do_itf.h"
#include "do_def.h"

/*!
 *  @defgroup Rop Production module
 *
 *  @brief Production task module
 *
 *  @details Production task is in charge of :
 *       @li RO Process management
 *  @{
 */


/* Declarations --------------------------------------------------------------*/

OsHandle_t hRopMboxHld;                   /*!< Production task mailbox */

static OsHandle_t hRopSemaList[ 3u ];   /*!< Production semaphores handle array */
/* Defines -------------------------------------------------------------------*/

#define ROP_MSG_SEM_IDX ((INT32)0)
#define ROP_TIM_SEM_IDX ((INT32)1)

/* Code ----------------------------------------------------------------------*/


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Initialized Production data
 *
 *  @details  Ensure all production data are initialized before production task start
 *
 */
static void InitData(void)
{
   eOsRc_t rc;
   static const PLAINCHAR* MBoxName = "ROP_MBX";

   static const PLAINCHAR* SemaNameArray [3u] =
   {
      "ROP_MSG_SEM",
      "ROP_TIM_SEM",
      NULL
   };

   /*! resolve seampahore ID to name  */
   rc=OS_SemaLookUpL(SemaNameArray, hRopSemaList );
   ASSERT( OS_GOOD == rc ,"Wrong OS_SemaLookUpL call");
   /*! resolve Mailbox  ID to name  */
   rc=OS_MboxLookUp( MBoxName, &hRopMboxHld );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxLookUp call");

   /*! Link MailBox to Semaphore    */
   rc=OS_MboxDefSema(MBoxName, SemaNameArray[ROP_MSG_SEM_IDX] );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxDefSema call");

   rop_readCanOpenSetpoints();

   // Initialize the last Ro Rejection
   DO_READ(ProductionProcessParameters.ProcessDescription.LastRoRejection, g_DoAlertAndAlarmsSetpoints.RoRejectionSp)

   /*! Initialize Production Process Parameters to force the Initialization */
   ProductionProcessParameters.ProcessDescription.ProcessMode = LAST_ROP_MODE;
   ProductionProcessParameters.ProcessDescription.ProcessSequence = LAST_ROP_SEQUENCE;
   ProductionProcessParameters.ProcessDescription.ProcessState = LAST_ROP_STATE;
   ProductionProcessParameters.TransitionDescription.TransitionType = LAST_ROP_TRANSITION;
   rop_setNewMode(&ProductionProcessParameters, ROP_MODE_INITIALIZATION, FALSE);

   /*! Get the Recorded Parameters in Flash */
   ProductionProcessParameters.pProcessRecord = (Rop_Record_t *)SYS_RestoreRecord (SYS_REC_PROD);
   
   DO_WRITE(g_DoProductionSetpoints.IsTheFirstStartUpProd, ProductionProcessParameters.pProcessRecord->isFirstStartUp); 
   g_bAppTaskInit++;
}


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Production task
 *
 *  @details   TBC
 *
 */
ATTRIBUTE_TASK void productionTask(void)
{
   while ( FALSE == g_isInitDone )
   {
      //OS_TaskYield();
      OS_TaskDelay(DELAY500MS);
   }


   InitData();

   SYSLOG(INFO, "Production task .........: START");

   while (g_isProductionTaskActive)
   {
      INT32       sdwSemaListOffset;
      ROP_Msg_t   ropMsg;

      sdwSemaListOffset = OS_SemaWaitLHI( (OsHandle_t*)hRopSemaList ) ;

      switch ( sdwSemaListOffset )
      {
         case ROP_MSG_SEM_IDX:
         {
            /*! Just get the message (does not wait for it) */
            MSG_GetWaitH( hRopMboxHld, (psMSG_t)&ropMsg, (UINT8)0U /* Get */ );
            /*! Analyses message content */
            ROP_MsgParser(&ropMsg);
            break;
         }

         case ROP_TIM_SEM_IDX:
         {
            rop_productionStateMachine(ROP_LAST_ORDER);
            break;
         }

         default:
         {
            ERROR("Wrong semaphore oof  receive in Production task %d",sdwSemaListOffset);
            break;
         }
      }
   }

   SYSLOG(INFO, "Production task .........: END");

   OS_TaskTerminate(OS_SELFTASK);
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

