/******************************************************************************
  MILLIPORE Bioscience LabWater copyright (c) 2010. All rights reserved.

   Version: 0.1

   Description: The  GPIO hal functions.

   Author(s): ELSYS-DESIGN

   History :
   CCR   | Date      | Version | Author       | Description
   =============================================================================
   | 02/05/2013|    0.1  | ELSYS-DESIGN | Creation
   -----------------------------------------------------------------------------
   |           |         |              |

 *****************************************************************************/
#include "stm32f4xx_it.h"
#include "drv_hal_uart.h"
#include "drv_hal_adc.h"
#include "drv_hal_gpio.h"
#include "drv_hal_ic.h"
#include "dil_timer.h"

/******************************************************************************
  EXTERNAL VARIABLES DECLARATIONS
 *****************************************************************************/

/******************************************************************************
  External ISR functions declaractions.
 *****************************************************************************/
//CanOpen Port interrupts
extern void CAN_TX_int (void);
extern void CAN_RX_int (void);
extern void CAN_Err_int(void);
#if defined D_BSP_PROCESS_BOARD && ( defined LOG_TO_CONSOLE || defined LOG_TO_CAN_AND_CONSOLE)
extern void ComAIsr(void);
#endif

/******************************************************************************
  INTERNAL VARIABLES DECLARATIONS
 *****************************************************************************/
DIL_UINT8 bCurAlternance   = 0;

DIL_UINT16 wTimeOutPSforADConv;
DIL_UINT16 wLastPeriodPS;

#ifdef __TESTBOARD__
DIL_UINT8 s_bTP1             = 0;
DIL_UINT8 s_bTP6             = 0;
DIL_UINT8 s_bTP7             = 0;
#endif

#ifdef _IT_ADC_OVR_
static DIL_UINT32 s_dwItOvrAdc1 = 0UL;
static DIL_UINT32 s_dwItOvrAdc2 = 0UL;
static DIL_UINT32 s_dwItOvrAdc3 = 0UL;
#endif
/******************************************************************************
  ISR declaractions.
 *****************************************************************************/
/*******************************************************************************
   function    : TmrIcIsr
   description : This function handles TIM2 in input capture mode
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void TmrIcIsr(void)
{
   // TIM2 - CH1
   if(SET == TIM_GetITStatus(TIM2, TIM_IT_CC1))
   {
      // Clear TIM2 Capture compare interrupt pending bit.
      TIM_ClearITPendingBit(TIM2, TIM_IT_CC1);
      addFlowCapture (1, TIM_GetCapture1(TIM2));
   }
   // TIM2 - CH2
   else if(SET == TIM_GetITStatus(TIM2, TIM_IT_CC2))
   {
      TIM_ClearITPendingBit(TIM2, TIM_IT_CC2);
      addFlowCapture (2, TIM_GetCapture2(TIM2));
   }
   // TIM2 - CH3
   else if(SET == TIM_GetITStatus(TIM2, TIM_IT_CC3))
   {
      TIM_ClearITPendingBit(TIM2, TIM_IT_CC3);
      addFlowCapture (3, TIM_GetCapture3(TIM2));
   }
   // TIM2 - CH4 : Power supply
   else if(SET == TIM_GetITStatus(TIM2, TIM_IT_CC4))
   {
      TIM_ClearITPendingBit(TIM2, TIM_IT_CC4);
      addPowerCapture (TIM_GetCapture4(TIM2));

      // calculation of current period in 0.1ms for checking
      // Min half period @ 64Hz -> half period : 7.8ms    -10% ->7.02ms
      readLastPeriodPowerCapture(&wLastPeriodPS);
      if(wLastPeriodPS > 70)
      {
        // start conversion AD
      drv_hal_adc_start_conv(ID_ADC_1, TRUE);
      drv_hal_adc_start_conv(ID_ADC_3, TRUE);
      bCurAlternance = 1 - bCurAlternance;
      drv_hal_adc_res_new_seq(bCurAlternance);
     }

     //set time out for Ad conversion
     // Max half period @ 47Hz -> half period : 10.6ms    +10% ->11.7s
     wTimeOutPSforADConv = 117;

#ifdef __TESTBOARD__
      s_bTP1 = 1 - s_bTP1;
      drv_hal_gpio_ioctl(DRV_HAL_GPIO_TEST_RFU1,
                         DIL_HAL_GPIO_CMD_SET_VALUE,
                         (DIL_PVOID)&s_bTP1,
                         sizeof(DIL_UINT8),
                         (DIL_PUINT32)NULL);
#endif
   }
}

/*******************************************************************************
   function    : TmrIsr (frequence 10 kHz)
   description : This function handles TIM7 global interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void TmrIsr(void)
{
   TIM7->SR = (unsigned short)(~(TIM_IT_Update));
   dil_timer_callBack();

   // check PS time out for AD conversion
   // time since last conversion
   if(0 == wTimeOutPSforADConv)
   {
      // time out reached -> start new conversion
      drv_hal_adc_start_conv(ID_ADC_1, TRUE);
      drv_hal_adc_start_conv(ID_ADC_3, TRUE);
      bCurAlternance = 1 - bCurAlternance;
      drv_hal_adc_res_new_seq(bCurAlternance);

      //set time out for Ad conversion
      // Max half period @ 47Hz -> half period : 10.6ms    +10% ->11.7s
      wTimeOutPSforADConv = 117-1;
   }
   else
   {
      wTimeOutPSforADConv--;
   }
}

/*******************************************************************************
   function    : Usart1Isr
   description : This function handles USART1 global interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Usart1Isr(void)
{
   DIL_UINT8 bByte = 0;
   if(RESET != USART_GetITStatus(USART1, USART_IT_RXNE))
   {
      bByte = ( USART_ReceiveData(USART1) & 0xFF );
      dil_hal_uart_receive_byte( DRV_UART_1, bByte);
   }

   if(RESET != USART_GetITStatus(USART1, USART_IT_TXE))
   {
      dil_hal_uart_send_byte_ISR( DRV_UART_1);
   }

   if(USART_GetFlagStatus(USART1, USART_FLAG_LBD))
   {
      USART_ClearFlag(USART1, USART_FLAG_LBD);
   }
}



/*******************************************************************************
   function    : Usart2Isr
   description : This function handles USART2 global interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Usart2Isr(void)
{
#if defined D_BSP_PROCESS_BOARD && defined INCLUDE_TOC && ( defined LOG_TO_CONSOLE  || defined LOG_TO_CAN_AND_CONSOLE)
   ComAIsr();
#else
   DIL_UINT8 bByte = 0;
   if(RESET != USART_GetITStatus(USART2, USART_IT_RXNE))
   {
      bByte = ( USART_ReceiveData(USART2) & 0xFF );
      dil_hal_uart_receive_byte( DRV_UART_2, bByte);
   }

   if(RESET != USART_GetITStatus(USART2, USART_IT_TXE))
   {
      dil_hal_uart_send_byte_ISR( DRV_UART_2);
   }

   if(USART_GetFlagStatus(USART2, USART_FLAG_LBD))
   {
      USART_ClearFlag(USART2, USART_FLAG_LBD);
   }
#endif
}



/*******************************************************************************
   function    : Usart3Isr
   description : This function handles USART3 global interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Usart3Isr(void)
{
#if defined D_BSP_STM3240GEVAL && ( defined LOG_TO_CONSOLE  || defined LOG_TO_CAN_AND_CONSOLE )
   ComAIsr();
#else
   DIL_UINT8 bByte = 0;
   if(RESET != USART_GetITStatus(USART3, USART_IT_RXNE))
   {
      bByte = ( USART_ReceiveData(USART3) & 0xFF );
      dil_hal_uart_receive_byte( DRV_UART_3, bByte);
   }

   if(RESET != USART_GetITStatus(USART3, USART_IT_TXE))
   {
      dil_hal_uart_send_byte_ISR( DRV_UART_3);
   }

   if(USART_GetFlagStatus(USART3, USART_FLAG_LBD))
   {
      USART_ClearFlag(USART3, USART_FLAG_LBD);
   }
#endif
}



/*******************************************************************************
   function    : Usart6Isr
   description : This function handles USART6 global interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Usart6Isr(void)
{
#if defined D_BSP_PROCESS_BOARD && !defined INCLUDE_TOC && ( defined LOG_TO_CONSOLE  || defined LOG_TO_CAN_AND_CONSOLE )
   ComAIsr();
#else
   DIL_UINT8 bByte = 0;
   if(RESET != USART_GetITStatus(USART6, USART_IT_RXNE))
   {
      bByte = ( USART_ReceiveData(USART6) & 0xFF );
      dil_hal_uart_receive_byte( DRV_UART_6, bByte);
   }

   if(RESET != USART_GetITStatus(USART6, USART_IT_TXE))
   {
      dil_hal_uart_send_byte_ISR( DRV_UART_6);
   }

   if(USART_GetFlagStatus(USART6, USART_FLAG_LBD))
   {
      USART_ClearFlag(USART6, USART_FLAG_LBD);
   }
#endif
}


/*******************************************************************************
   function    : Can1TxIsr
   description : This function handles can1 TX interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Can1TxIsr(void)
{
   CAN_TX_int();
} /* void CAN1_TX_IRQHandler() */


/*******************************************************************************
   function    : Can1Rx0Isr
   description : This function handles can1 RX0 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Can1Rx0Isr(void)
{
   CAN_RX_int();
} /* void CAN1_RX0_IRQHandler() */


/*******************************************************************************
   function    : Can1Rx1Isr
   description : This function handles can1 RX1 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Can1Rx1Isr(void)
{
   CAN_RX_int();
} /* void CAN1_RX1_IRQHandler() */



/*******************************************************************************
   function    : Can1SceIsr
   description : This function handles can1 ERR interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Can1SceIsr(void)
{
   CAN_Err_int();
} /* void CAN1_SCE_IRQHandler() */



/*******************************************************************************
   function    : Exti0_Isr
   description : This function handles EXTI0 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti0_Isr(void)
{
   if(RESET != EXTI_GetITStatus(EXTI_Line0))
   {
      /* Clear the EXTI line 0 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line0 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(0);
   }
}

/*******************************************************************************
   function    : Exti1_Isr
   description : This function handles EXTI1 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti1_Isr(void)
{
   if(RESET != EXTI_GetITStatus(EXTI_Line1))
   {
      /* Clear the EXTI line 1 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line1 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(1);
   }
}

/*******************************************************************************
   function    : Exti2_Isr
   description : This function handles EXTI2 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti2_Isr(void)
{
   if(RESET != EXTI_GetITStatus(EXTI_Line2))
   {
      /* Clear the EXTI line 2 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line2 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(2);
   }
}

/*******************************************************************************
   function    : Exti3_Isr
   description : This function handles EXTI3 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti3_Isr(void)
{
   if(RESET != EXTI_GetITStatus(EXTI_Line3))
   {
      /* Clear the EXTI line 3 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line3 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(3);
   }
}

/*******************************************************************************
   function    : Exti4_Isr
   description : This function handles EXTI4 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti4_Isr(void)
{
   if(RESET != EXTI_GetITStatus(EXTI_Line4))
   {
      /* Clear the EXTI line 4 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line4 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(4);
   }
}

/*******************************************************************************
   function    : Exti5_9_Isr
   description : This function handles EXTI5 to EXTI9 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti5_9_Isr(void)
{
   /* Line 5 */
   if(RESET != EXTI_GetITStatus(EXTI_Line5))
   {
      /* Clear the EXTI line 5 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line5 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(5);
   }
   /* Line 6 */
   if(RESET != EXTI_GetITStatus(EXTI_Line6))
   {
      /* Clear the EXTI line 6 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line6 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(6);
   }
   /* Line 7 */
   if(RESET != EXTI_GetITStatus(EXTI_Line7))
   {
      /* Clear the EXTI line 7 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line7 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(7);
   }
   /* Line 8 */
   if(RESET != EXTI_GetITStatus(EXTI_Line8))
   {
      /* Clear the EXTI line 8 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line8 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(8);
   }
   /* Line 9 */
   if(RESET != EXTI_GetITStatus(EXTI_Line9))
   {
      /* Clear the EXTI line 9 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line9 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(9);
   }
}



/*******************************************************************************
   function    : Exti10_15_Isr
   description : This function handles EXTI10 to EXTI15 interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void Exti10_15_Isr(void)
{
   /* Line 10 */
   if(RESET != EXTI_GetITStatus(EXTI_Line10))
   {
      /* Clear the EXTI line 10 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line10 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(10);
   }
   /* Line 11 */
   if(RESET != EXTI_GetITStatus(EXTI_Line11))
   {
      /* Clear the EXTI line 11 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line11 );
      // This line is special, it is used to trigger the ADC2 for the
      // resistivity DIL. As such, it is not linked to the HAL GPIO
   }
   /* Line 12 */
   if(RESET != EXTI_GetITStatus(EXTI_Line12))
   {
      /* Clear the EXTI line 12 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line12 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(12);
   }
   /* Line 13 */
   if(RESET != EXTI_GetITStatus(EXTI_Line13))
   {
      /* Clear the EXTI line 13 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line13 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(13);
   }
   /* Line 14 */
   if(RESET != EXTI_GetITStatus(EXTI_Line14))
   {
      /* Clear the EXTI line 14 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line14 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(14);
   }
   /* Line 15 */
   if(RESET != EXTI_GetITStatus(EXTI_Line15))
   {
      /* Clear the EXTI line 15 pending bit */
      EXTI_ClearITPendingBit( EXTI_Line15 );
      /* and call the corresponding callback */
      drv_hal_gpio_callback(15);
   }
}

/*******************************************************************************
   function    : AdcIsr
   description : This function handles ADC interrupt request.
   Parameters  :
      None
   Returns:
      None

 ******************************************************************************/
void AdcIsr( void )
{
   DIL_UINT16 wConvValue = 0;
   if(SET == ADC_GetITStatus(ADC1, ADC_IT_EOC))
   {
      wConvValue = ADC_GetConversionValue(ADC1);
      drv_hal_adc_addValue( ID_ADC_1, wConvValue);
 #ifdef __TESTBOARD__
      s_bTP6 = 1 - s_bTP6;
      drv_hal_gpio_ioctl(DRV_HAL_GPIO_TEST_RFU6,
                            DIL_HAL_GPIO_CMD_SET_VALUE,
                            (DIL_PVOID)&s_bTP6,
                            sizeof(DIL_UINT8),
                            (DIL_PUINT32)NULL);
#endif
   }
   if(SET == ADC_GetITStatus(ADC2, ADC_IT_EOC))
   {
      wConvValue = ADC_GetConversionValue(ADC2);
      // The resistivity needs one more step to be synchronized on power supply
      // interrupt and EXTI_11
      drv_hal_adc_res_new_measure( wConvValue);
   }
   if(SET == ADC_GetITStatus(ADC3, ADC_IT_EOC))
   {
      wConvValue = ADC_GetConversionValue(ADC3);
      drv_hal_adc_addValue( ID_ADC_3, wConvValue);
#ifdef __TESTBOARD__
      s_bTP7 = 1 - s_bTP7;
      drv_hal_gpio_ioctl(DRV_HAL_GPIO_TEST_RFU6,
                            DIL_HAL_GPIO_CMD_SET_VALUE,
                            (DIL_PVOID)&s_bTP7,
                            sizeof(DIL_UINT8),
                            (DIL_PUINT32)NULL);
#endif
   }

#ifdef _IT_ADC_OVR_
   if(SET == ADC_GetITStatus(ADC1, ADC_IT_OVR))
   {
      s_dwItOvrAdc1++;
      ADC_ClearITPendingBit(ADC1, ADC_IT_OVR);
      ADC_Cmd( ADC1, DISABLE);
      ADC_Cmd( ADC1, ENABLE);
      drv_hal_adc_start_conv( ID_ADC_1, FALSE);
   }
   if(SET == ADC_GetITStatus(ADC2, ADC_IT_OVR))
   {
      s_dwItOvrAdc2++;
      ADC_ClearITPendingBit(ADC2, ADC_IT_OVR);
   }
   if(SET == ADC_GetITStatus(ADC3, ADC_IT_OVR))
   {
      s_dwItOvrAdc3++;
      ADC_ClearITPendingBit(ADC3, ADC_IT_OVR);
      ADC_Cmd( ADC3, DISABLE);
      ADC_Cmd( ADC3, ENABLE);
      drv_hal_adc_start_conv( ID_ADC_3, FALSE);
   }
#endif
}
